package com.bocloud.cop.booter.config;

import java.util.Map;

import org.springframework.amqp.core.AcknowledgeMode;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.core.TopicExchange;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import com.bocloud.cop.booter.listener.TaskHostListener;
import com.bocloud.cop.booter.listener.WorkerLogListener;
import com.megatron.common.utils.MapTools;

/**
 * rabbitmq相关配置
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {

    @Autowired
    private ConnectionFactory connectionFactory;
    private Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");

    /**
     * 声明补丁部署队列
     *
     * @return
     */
    @Bean
    public Queue declarePatchQueue() {
        return new Queue("task.job.patch.result", true, false, false, arguments);
    }

    /**
     * 声明合规检查队列
     *
     * @return
     */
    @Bean
    public Queue declareComplianceQueue() {
        return new Queue("task.job.compliance.result", true, false, false, arguments);
    }

    /**
     * 声明巡检任务队列
     *
     * @return
     */
    @Bean
    public Queue declareInspectionQueue() {
        return new Queue("task.job.inspection.result", true, false, false, arguments);
    }

    /**
     * 声明延时任务队列
     *
     * @return
     */
    @Bean
    public Queue declareDelayQueue() {
        return new Queue("task.job.patch.delay", true, false, false, arguments);
    }

    /**
     * 主机资源增量同步队列
     *
     * @return
     */
    @Bean
    public Queue declareTaskHostOperateQueue() {
        return new Queue("bocloud.taskhost.operate", true, false, false, arguments);
    }

    /**
     * 定义direct exchange
     *
     * @return
     */
    @Bean
    public DirectExchange declareDirectExchange() {
        return new DirectExchange("bocloud.direct.exchange", true, true, arguments);
    }

    /**
     * 定义topic exchange
     *
     * @return
     */
    @Bean
    public TopicExchange declareTopicExchange() {
        return new TopicExchange("bocloud.topic.exchange", true, true, arguments);
    }

    /**
     * 声明作业日志队列
     *
     * @return
     */
    @Bean
    public Queue declareWorkerLogQueue() {
        return new Queue("worker.running.log.queue", true, false, false, arguments);
    }

    /**
     * 绑定作业日志队列
     *
     * @return
     */
    @Bean
    public Binding bindingWorkerLogQueue() {
        return BindingBuilder.bind(declareWorkerLogQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 补丁部署队列绑定到bocloud.direct.exchange
     *
     * @return
     */
    @Bean
    public Binding bindingPatchQueue() {
        return BindingBuilder.bind(declarePatchQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 合规检查队列绑定到bocloud.direct.exchange
     *
     * @return
     */
    @Bean
    public Binding bindingComplianceQueue() {
        return BindingBuilder.bind(declareComplianceQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 巡检任务队列绑定到bocloud.direct.exchange
     *
     * @return
     */
    @Bean
    public Binding bindingInspectQueue() {
        return BindingBuilder.bind(declareInspectionQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     * 延时任务队列绑定bocloud.direct.exchange
     *
     * @return
     */
    @Bean
    public Binding bindingDelayQueue() {
        return BindingBuilder.bind(declareDelayQueue()).to(declareDirectExchange()).withQueueName();
    }

    /**
     *
     *
     * @return
     */
    @Bean
    public Binding bindingTaskHostOperateQueue() {
        return BindingBuilder.bind(declareTaskHostOperateQueue()).to(declareDirectExchange()).withQueueName();
    }

    @Bean
    public SimpleMessageListenerContainer autoTaskHostListener(TaskHostListener taskHostListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareTaskHostOperateQueue());
        container.setAcknowledgeMode(AcknowledgeMode.AUTO); // 设置确认模式自动确认
        container.setMessageListener(taskHostListener);
        return container;
    }

    @Bean
    public SimpleMessageListenerContainer autoWorkerLogListener(WorkerLogListener workerLogListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareWorkerLogQueue());
        container.setAcknowledgeMode(AcknowledgeMode.AUTO); // 设置确认模式自动确认
        container.setMessageListener(workerLogListener);
        return container;
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        amqpTemplate.setExchange(declareDirectExchange().getName());
        return amqpTemplate;
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitAdmin rabbitAdmin() {
        RabbitAdmin rabbitAdmin = new RabbitAdmin(connectionFactory);
        return rabbitAdmin;
    }

}
