package com.bocloud.cop.booter.config;

import com.bocloud.cmp.boot.config.PagerArgumentResolver;
import com.bocloud.cmp.boot.interceptor.AccessInterceptor;
import com.bocloud.cmp.boot.interceptor.LogInterceptor;
import com.fasterxml.jackson.annotation.JsonTypeName;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.MapperFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.megatron.framework.core.InterceptorBridge;
import com.megatron.framework.core.config.InterceptorConfiguration;
import com.megatron.framework.core.domain.InterceptorEntity;
import lombok.extern.slf4j.Slf4j;
import org.reflections.Reflections;
import org.reflections.scanners.Scanner;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.converter.ByteArrayHttpMessageConverter;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.stereotype.Component;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;

import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Set;

/**
 * 拦截器配置类
 *
 * @author dmw
 */
@Slf4j
@Component
public class WebInterceptorConfiguration implements InterceptorConfiguration {

    private static final String[] ACCESS_EXCLUDES = new String[]{"/status", "/status/db", "/status/**", "/v1/alistar/**",
            "/v1/status/**", "/login", "/sso", "/v1/logview", "/v1/logzip", "/v1/config/**", "/v1/plugins/**", "/*/*.css",
            "/*/*.js", "/*/*.png", "/*/*.jpg", "/*/*.jpeg", "/*.html", "/*/*.html", "/swagger-resources/**", "/favicon.ico",
            "/webjars/**", "/v3/**", "/swagger-ui.html/**", "**/api-docs", "/error", "/favicon.ico" };
    private static final String[] LOG_EXCLUDES = new String[]{"/*/*.css", "/*/*.js", "/*/*.png", "/*/*.jpg",
            "/*/*.jpeg", "/*.html", "/*/*.html", "/swagger-resources/**", "/favicon.ico", "/webjars/**", "/v3/**",
            "/swagger-ui.html/**", "**/api-docs", "/error", "/favicon.ico" };
    private static final String[] INCLUDES = new String[]{"/**"};
    private final LogInterceptor logInterceptor;
    private final AccessInterceptor accessInterceptor;

    @Autowired
    public WebInterceptorConfiguration(AccessInterceptor accessInterceptor, LogInterceptor logInterceptor) {
        this.logInterceptor = logInterceptor;
        this.accessInterceptor = accessInterceptor;
    }

    @Override
    public InterceptorBridge buildBridge() {
        log.info("start to build interceptor bridge ...");

        InterceptorEntity accessEntity = InterceptorEntity.builder().interceptor(accessInterceptor).includes(INCLUDES).excludes(ACCESS_EXCLUDES).order(0).build();
        InterceptorEntity auditLogEntity = InterceptorEntity.builder().interceptor(logInterceptor).includes(INCLUDES).excludes(LOG_EXCLUDES).order(1).build();

        List<InterceptorEntity> interceptors = new ArrayList<>();
        interceptors.add(accessEntity);
        interceptors.add(auditLogEntity);

        List<HttpMessageConverter<?>> converters = new ArrayList<>();
        converters.add(this.byteConverter());
        converters.add(this.jsonConverter());

        List<HandlerMethodArgumentResolver> resolvers = new ArrayList<>();
        resolvers.add(new PagerArgumentResolver());

        InterceptorBridge bridge = new InterceptorBridge();
        bridge.setInterceptors(interceptors);
        bridge.setResolvers(resolvers);
        bridge.setMessageConverters(converters);

        log.info("build interceptor bridge success!!!");
        return bridge;
    }

    private HttpMessageConverter<?> byteConverter() {
        return new ByteArrayHttpMessageConverter();
    }

    private HttpMessageConverter<?> jsonConverter() {
        ObjectMapper objectMapper = new ObjectMapper();
        objectMapper.configure(MapperFeature.ACCEPT_CASE_INSENSITIVE_ENUMS, true);
        objectMapper.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false);
        Reflections reflections = new Reflections("com", new Scanner[0]);
        Set<Class<?>> classSet = reflections.getTypesAnnotatedWith(JsonTypeName.class);
        objectMapper.registerSubtypes(classSet);
        objectMapper.setDateFormat(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss"));

        MappingJackson2HttpMessageConverter jackson2HttpMessageConverter = new MappingJackson2HttpMessageConverter();
        jackson2HttpMessageConverter.setDefaultCharset(Charset.defaultCharset());
        jackson2HttpMessageConverter.setObjectMapper(objectMapper);
        jackson2HttpMessageConverter.setPrettyPrint(true);
        jackson2HttpMessageConverter.setSupportedMediaTypes(Collections.singletonList(MediaType.APPLICATION_JSON));
        return jackson2HttpMessageConverter;
    }
}
