package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.megatron.common.model.GeneralResult;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpResponse;
import org.apache.http.auth.AuthScope;
import org.apache.http.auth.Credentials;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.conn.ssl.NoopHostnameVerifier;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;

import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509ExtendedTrustManager;
import java.io.IOException;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.net.Socket;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.*;
import java.util.stream.Collectors;

@Tag(name = "监控系统对接")
@RestController
@RequestMapping("/v1/alistar")
@Slf4j
public class AlistarController {

    @Value("${alistar.url:http://127.0.0.1/alistar}")
    private String alistarUrl;
    @Value("${alistar.username:admin}")
    private String alistarUsername;
    @Value("${alistar.password:password}")
    private String alistarPassword;

    @GetMapping("/cmdb/{category}")
    @Operation(summary = "为CMDB查询ALISTAR数据")
    public GeneralResult<List<Map<String, Object>>> alistar(
            @PathVariable(value = "category") String category,
            @RequestParam(value = "category", required = false) String alistarCategory,
            @RequestParam(value = "uniform", required = false) String alistarUniform
    ) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        final String[] safeUniforms = new String[]{
                "device.firewall", // 防火墙
                "device.ngfw", // 下一代防火墙
                "device.ips", // IPS入侵防御
                "device.ids", // IDS入侵检测
                "device.waf", // Web应用防火墙
                "device.scanner", // 漏洞扫描
                "device.utm", // 统一威胁管理UTM
                "device.acm", // 上网行为管理
                "device.antivirus" // 防病毒
        };
        List<Map<String, Object>> list = new ArrayList<>();
        List<AlistarInstance> instances = this.getAlistarResourceInstance();
        if ("SERVER".equalsIgnoreCase(category)) {
            instances = instances.stream().filter(inst -> "服务器".equals(inst.getCategory())).collect(Collectors.toList());
        } else if ("NET".equalsIgnoreCase(category)) {
            instances = instances.stream()
                    .filter(inst -> "网络设备".equals(inst.getCategory()))
                    .filter(inst -> !Arrays.asList(safeUniforms).contains(inst.getUniform()))
                    .collect(Collectors.toList());
        } else if ("SAFE".equalsIgnoreCase(category)) {
            instances = instances.stream()
                    .filter(inst -> "网络设备".equals(inst.getCategory()))
                    .filter(inst -> Arrays.asList(safeUniforms).contains(inst.getUniform()))
                    .collect(Collectors.toList());
        } else if ("ALL".equalsIgnoreCase(category)) {
            // do nothing
        } else {
            instances = new ArrayList<>();
        }
        if (StringUtils.isNotEmpty(alistarCategory)) {
            instances = instances.stream()
                    .filter(inst -> Arrays.asList(alistarCategory.split(",")).contains(inst.getCategory()))
                    .collect(Collectors.toList());
        }
        if (StringUtils.isNotEmpty(alistarUniform)) {
            instances = instances.stream()
                    .filter(inst -> Arrays.asList(alistarUniform.split(",")).contains(inst.getUniform()))
                    .collect(Collectors.toList());
        }
        for (AlistarInstance instance : instances) {
            Map<String, Object> item = JSON.parseObject(JSON.toJSONString(instance));
            for (int i = 0; i < 10; i++) {
                item.put("bocloud.zero." + (i + 1), 0);
            }
            item.put("instanceUuid", DigestUtils.md5Hex("ALISTAR-" + instance.getInstanceId()));
            List<AlistarInstanceMetricValue> values = this.getAlistarResourceInstanceMetricValue(instance);
            for (AlistarInstanceMetricValue value : values) {
                if (value.getKey().startsWith("os.memory.") || value.getKey().startsWith("memory.")) {
                    BigDecimal valueGB = new BigDecimal(value.getValue()).divide(BigDecimal.valueOf(1024L * 1024L * 1024L), 3, RoundingMode.HALF_UP);
                    item.put(value.getKey(), valueGB.toPlainString());
                } else if (value.getKey().startsWith("capacity.")) {
                    BigDecimal valueGB = new BigDecimal(value.getValue()).divide(BigDecimal.valueOf(1024L * 1024L * 1024L * 1024L), 3, RoundingMode.HALF_UP);
                    item.put(value.getKey(), valueGB.toPlainString());
                } else if (value.getKey().startsWith("net.interface.") ||
                        value.getKey().startsWith("os.service.") ||
                        value.getKey().startsWith("os.processor.") ||
                        value.getKey().startsWith("os.process.")) {
                    // do nothing
                } else {
                    item.put(value.getKey(), value.getValue());
                }
            }
            list.add(item);
        }
        return new GeneralResult<>(true, list, "查询成功");
    }

    private List<AlistarInstanceMetricValue> getAlistarResourceInstanceMetricValue(AlistarInstance instance) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String url = alistarUrl + "/api/v1/resource/instance/metric/value";
        Map<String, Object> httpParam = new HashMap<>();
        httpParam.put("parentId", instance.getInstanceId());
        String httpBody = JSON.toJSONString(httpParam);
        HttpPost httpPost = new HttpPost(url);
        httpPost.setEntity(new StringEntity(httpBody, ContentType.APPLICATION_JSON));
        log.info("curl -k -X POST --user '{}:{}' '{}' -d '{}'", alistarUsername, alistarPassword, url, httpBody);
        HttpResponse httpResponse = this.getHttpClient().execute(httpPost);
        String httpResult = this.getAlistarData(httpResponse);
        return JSON.parseArray(httpResult, AlistarInstanceMetricValue.class);
    }

    private List<AlistarInstanceMetric> getAlistarResourceInstanceMetric(AlistarInstance instance) throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String url = alistarUrl + "/api/v1/resource/instance/" + instance.getInstanceId() + "/metric" +
                "?uniform=" + URLEncoder.encode(instance.getUniform(), StandardCharsets.UTF_8);
        HttpGet httpGet = new HttpGet(url);
        log.info("curl -k --user '{}:{}' '{}'", alistarUsername, alistarPassword, url);
        HttpResponse httpResponse = this.getHttpClient().execute(httpGet);
        String httpBody = this.getAlistarData(httpResponse);
        return JSON.parseArray(httpBody, AlistarInstanceMetric.class);
    }

    private List<AlistarInstance> getAlistarResourceInstance() throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String url = alistarUrl + "/api/v1/resource/instance";
        HttpGet httpGet = new HttpGet(url);
        log.info("curl -k --user '{}:{}' '{}'", alistarUsername, alistarPassword, url);
        HttpResponse httpResponse = this.getHttpClient().execute(httpGet);
        String httpBody = this.getAlistarData(httpResponse);
        return JSON.parseArray(httpBody, AlistarInstance.class);
    }

    private List<AlistarUniform> getAlistarResourceUniform() throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String url = alistarUrl + "/api/v1/resource/uniform";
        HttpGet httpGet = new HttpGet(url);
        log.info("curl -k --user '{}:{}' '{}'", alistarUsername, alistarPassword, url);
        HttpResponse httpResponse = this.getHttpClient().execute(httpGet);
        String httpBody = this.getAlistarData(httpResponse);
        return JSON.parseArray(httpBody, AlistarUniform.class);
    }

    private List<AlistarCategory> getAlistarResourceCategory() throws NoSuchAlgorithmException, KeyManagementException, IOException {
        String url = alistarUrl + "/api/v1/resource/category";
        HttpGet httpGet = new HttpGet(url);
        log.info("curl -k --user '{}:{}' '{}'", alistarUsername, alistarPassword, url);
        HttpResponse httpResponse = this.getHttpClient().execute(httpGet);
        String httpBody = this.getAlistarData(httpResponse);
        return JSON.parseArray(httpBody, AlistarCategory.class);
    }

    private String getAlistarData(HttpResponse response) throws IOException {
        String body = EntityUtils.toString(response.getEntity());
        Assert.isTrue(response.getStatusLine().getStatusCode() == 200, body);
        JSONObject httpObject = JSON.parseObject(body);
        Integer code = httpObject.getInteger("code");
        String message = httpObject.getString("message");
        Assert.isTrue(code == 200, message);
        return httpObject.getString("data");
    }

    private HttpClient getHttpClient() throws NoSuchAlgorithmException, KeyManagementException {
        SSLContext sslContext = SSLContext.getInstance("TLS");
        sslContext.init(null, new TrustManager[]{new TrustAllManager()}, null);
        CredentialsProvider provider = new BasicCredentialsProvider();
        Credentials credentials = new UsernamePasswordCredentials(alistarUsername, alistarPassword);
        provider.setCredentials(AuthScope.ANY, credentials);
        return HttpClients.custom()
                .setSSLContext(sslContext)
                .setSSLHostnameVerifier(NoopHostnameVerifier.INSTANCE)
                .setDefaultCredentialsProvider(provider)
                .build();
    }

    private static class TrustAllManager extends X509ExtendedTrustManager {
        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType, Socket socket) throws CertificateException {

        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType, Socket socket) throws CertificateException {

        }

        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType, SSLEngine engine) throws CertificateException {

        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType, SSLEngine engine) throws CertificateException {

        }

        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {

        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {

        }

        @Override
        public X509Certificate[] getAcceptedIssuers() {
            return new X509Certificate[]{};
        }
    }

    @Data
    private static class AlistarCategory {
        private String name;
    }

    @Data
    private static class AlistarUniform {
        private String key;
        private String parentKey;
        private String name;
        private String category;
    }

    @Data
    private static class AlistarInstance {
        private Long instanceId;
        private String name;
        private String host;
        private String category;
        private String uniform;
    }

    @Data
    private static class AlistarInstanceMetric {
        private String key; // 指标Key
        private String name; // 指标名称
        private String type; // 指标类型，1为信息指标，2为性能指标
        private String valueType; // 指标值类型，1为整型，2为浮点型，3为字符，4为布尔型，5为Unix时间戳，6为运行时长
    }

    @Data
    private static class AlistarInstanceMetricValue {
        private String key; // 指标Key
        private String uniqueName; // 唯一名称
        private String valueType; // 指标值类型，1为整型，2为浮点型，3为字符，4为布尔型，5为Unix时间戳，6为运行时长
        private Long fetchTime; // 值采集时间
        private String value; // 当前采集值
        private Long lastFetchTime; // 上次值采集时间
        private String lastValue; // 上次采集值
    }

}
