package com.bocloud.cop.booter.controller;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Identification;
import com.bocloud.cop.intf.IdentificationService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/v1/identifications")
@Tag(name = "认证用户管理")
@Slf4j
public class AuthenController {

    @Autowired
    private IdentificationService identService;

    /**
     * 获取认证列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "认证用户列表")
    public GeneralResult<GridBean<Identification>> list(Pager pager) {
        return identService.list(pager);
    }

    @GetMapping("/simple")
    @Operation(tags = {"CMC", "CSC"}, summary = "认证用户列表（简单）")
    public GeneralResult<GridBean<Identification>> simple(Pager pager) {
        return identService.simple(pager);
    }

    /**
     * 添加认证
     *
     * @param params
     * @param context
     * @return
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "认证用户创建")
    public GeneralResult<Void> create(@RequestBody Identification params,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return identService.create(params, context.getTarget());
    }

    /**
     * 修改认证
     *
     * @param params
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "认证用户修改")
    public GeneralResult<Void> modify(@RequestBody Identification params, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        params.setId(id);
        return identService.modify(params, context.getTarget());
    }

    /**
     * 删除认证
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "认证用户删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return identService.remove(id, context);
    }

    /**
     * 认证详情
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "认证用户详情")
    public GeneralResult<Identification> detail(@PathVariable(value = Common.ID) Long id) {
        return identService.detail(id);
    }

    /**
     * 认证账号批量导入(账号名称+用户名唯一，相同更新，没有新增)
     *
     * @param request 请求
     * @param context 用户信息
     * @return 批量导入结果
     * @throws IOException io异常
     */
    @PostMapping(value = "/upload")
    @Operation(tags = {"CMC", "CSC"}, summary = "认证账号批量导入")
    public GeneralResult<Void> importExcel(HttpServletRequest request,
                                           @Value(Common.REQ_CONTEXT) RequestContext context) {
        try {
            MultipartHttpServletRequest multipartHttpServletRequest = (MultipartHttpServletRequest) request;
            MultiValueMap<String, MultipartFile> multiFileMap = multipartHttpServletRequest.getMultiFileMap();
            MultipartFile item = null;
            if (multiFileMap.values().size() > 0) {
                List<MultipartFile> multiFileList = multiFileMap.values().iterator().next();
                item = multiFileList.get(0);
            }
            if (null == item) {
                return new GeneralResult<>(false, "文件错误");
            }
            return identService.importExcel(item, context);
        } catch (Exception e) {
            log.error("上传失败", e);
        }
        return new GeneralResult<>(false, "文件错误");
    }

    /**
     * 导入模板下载
     *
     * @param params   下载参数
     * @param request  请求
     * @param response 返回
     */
    @GetMapping(value = "/template/download")
    @Operation(tags = {"CMC", "CSC"}, summary = "下载导入文件模板")
    public void download(@RequestParam(value = Common.PARAMS, required = false) String params,
                         HttpServletRequest request, HttpServletResponse response) {
        Map<String, Object> paramsMap = new HashMap<>();
        if (null != params) {
            JSONObject.parseObject(params, HashMap.class);
        }
        GeneralResult<?> result = identService.download(paramsMap);
        if (result.isFailed()) {
            return;
        }
        try {
            @SuppressWarnings("unchecked")
            Map<String, Object> map = JSONObject.parseObject(JSONObject.toJSONString(result.getData()), HashMap.class);
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/vnd.ms-excel");
            String fileName = map.get("fileName").toString();
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            String content = (String) map.get("data");
            if (content != null) {
                byte[] bytes = Base64.getDecoder().decode(content);
                os.write(bytes);
                os.close();
                os.flush();
            }
        } catch (Exception e) {
            log.error("下载失败");
        }
    }
}
