package com.bocloud.cop.booter.controller.module;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.module.NodeModule;
import com.bocloud.cop.entity.module.TaskNodeModuleParam;
import com.bocloud.cop.intf.NodeModuleService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.List;

/**
 * 模型控制器
 */
@Slf4j
@RestController
@RequestMapping("/v1/tasknodes/modules")
@Tag(name = "模型管理")
public class NodeModuleController {

    @Autowired
    private NodeModuleService nodeModuleService;

    /**
     * 查看模型列表
     *
     * @param pager
     * @return
     */
    @GetMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "模型列表")
    public GeneralResult<GridBean<NodeModule>> list(Pager pager) {
        return nodeModuleService.list(pager);
    }

    /**
     * 查看模型详情
     *
     * @param code
     * @return
     */
    @GetMapping(value = "/{code}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型详情")
    public GeneralResult<NodeModule> detailByCode(@PathVariable("code") String code) {
        return nodeModuleService.detailByCode(code);
    }

    /**
     * 添加模型
     *
     * @param params
     * @param context
     * @return 添加结果
     */
    @PostMapping
    @Operation(tags = {"CMC", "CSC"}, summary = "模型创建")
    public GeneralResult<Void> create(@RequestBody Object params, @Value(Common.REQ_CONTEXT) RequestContext context) {
        NodeModule module = JSONObject.parseObject(JSONObject.toJSONString(params), NodeModule.class);
        module.setCreatorId(context.getTarget());
        module.setMenderId(context.getTarget());
        return nodeModuleService.create(module);
    }

    /**
     * 修改模型
     *
     * @param params
     * @param id
     * @param context
     * @return
     */
    @PutMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型修改")
    public GeneralResult<Void> modify(@RequestBody NodeModule params, @PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nodeModuleService.modify(params, context.getTarget());
    }

    /**
     * 移除
     *
     * @param id
     * @param context
     * @return 操作结果
     */
    @DeleteMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型删除")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id,
                                      @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nodeModuleService.remove(id, context.getTarget());
    }

    @PutMapping(value = "/{id}/setting")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型默认参数设置")
    public GeneralResult<Void> setting(@PathVariable(value = Common.ID) Long id,
                                       @RequestBody List<TaskNodeModuleParam> params, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nodeModuleService.settingDefaultParams(id, params);
    }

    @GetMapping(value = "/{code}/default/info")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型默认参数详情")
    public GeneralResult<List<TaskNodeModuleParam>> defaultInfo(@PathVariable("code") String code,
                                                                @Value(Common.REQ_CONTEXT) RequestContext context) {
        return nodeModuleService.defaultInfo(code);
    }

    @PatchMapping(value = "/{id}")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型上/下架")
    public GeneralResult<Void> updateStatus(@PathVariable(value = Common.ID) Long id, @RequestBody JSONObject params,
                                            @Value(Common.REQ_CONTEXT) RequestContext context) {
        String status = params.getString("status");
        return nodeModuleService.updateStatus(id, status, context);
    }

    @GetMapping(value = "/{id}/export")
    @Operation(tags = {"CMC", "CSC"}, summary = "模型导出")
    public void export(@PathVariable(value = Common.ID) Long id, HttpServletRequest request,
                       HttpServletResponse response) {
        GeneralResult<?> result = nodeModuleService.detail(id);
        if (result.isFailed()) {
            return;
        }
        String content = JSONObject.toJSONString(result.getData());
        response.setCharacterEncoding("utf-8");
        response.setContentType("multipart/form-data");
        try {
            String fileName = "task_" + System.currentTimeMillis() + ".txt";
            final String userAgent = request.getHeader("user-agent").toLowerCase();
            if (userAgent.contains("trident") || userAgent.contains("edge")) {
                fileName = URLEncoder.encode(fileName, "UTF-8");
            } else {
                fileName = new String(fileName.getBytes("utf-8"), "iso-8859-1");
            }
            response.setHeader("Content-Disposition", "attachment;fileName=" + fileName);
            OutputStream os = response.getOutputStream();
            byte[] bytes = content.getBytes();
            os.write(bytes);
            os.close();
            os.flush();
        } catch (Exception e) {
            log.error("Download fail:", e);
        }
    }
}
