package com.bocloud.cop.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.core.WorkerCache;
import com.bocloud.cop.core.WorkerDaemon;
import com.bocloud.cop.core.WorkerStrategyCache;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.entity.TaskWorker.Category;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;
import com.megatron.framework.registry.ZooKeeperClient;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.framework.recipes.cache.TreeCache;
import org.apache.curator.framework.recipes.cache.TreeCacheEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * Worker监听器
 *
 * @author dmw
 */
@Slf4j
@Component
public class WorkerListener {

    private static final Service SERVICE = Service.create(BoCloudService.GAIA.getName());
    private final ZooKeeperClient client;
    private final CurrentService service;
    private final TaskWorkerRepository workerRepository;
    private final String workerPath;
    private final WorkerCache workerCache;
    private final WorkerStrategyCache workerStrategyCache;

    private TreeCache treeCache;

    @Autowired
    public WorkerListener(ZooKeeperClient client, CurrentService service, TaskWorkerRepository workerRepository,
        WorkerCache workerCache, WorkerStrategyCache workerStrategyCache) {
        this.client = client;
        this.service = service;
        this.workerRepository = workerRepository;
        this.workerPath = service.getService().getPrefix() + "/gaia";
        this.workerCache = workerCache;
        this.workerStrategyCache = workerStrategyCache;
    }

    @PostConstruct
    void start() {
        this.refreshWorker();
        try {
            treeCache = new TreeCache(this.client.getClient(), workerPath);
            treeCache.start();
            treeCache.getListenable().addListener((client, event) -> {
                if (Objects.nonNull(event.getData())) {
                    refreshWorker(event);
                } else {
                    refreshWorker();
                }
            });
            //定时任务-定时去获取zk中gaia服务所对应节点的数据并对缓存和数据库做相应处理
            this.service.getScheduledExecutorService().scheduleAtFixedRate(
                new WorkerDaemon(client, workerPath, workerRepository, workerCache), 10L, 60L, TimeUnit.SECONDS);
        } catch (Exception e) {
            log.error("add worker listener exception!", e);
        }
    }

    @PreDestroy
    void destroy() {
        if (null != this.treeCache) {
            this.treeCache.close();
        }
    }

    /**
     * 刷新Worker信息
     *
     * @throws Exception
     *             系统异常
     */
    private void refreshWorker() {
        log.info("leader 刷新worker信息，当前服务是否是leader：{}", service.isLeader());
        if (!service.isLeader()) {
            return;
        }
        try {
            List<String> instances = this.client.getChildren(workerPath);
            instances.forEach(instance -> {
                // 首先判断当前worker是否存在, 以instance作为索引进行查找
                try {
                    String instancePath = this.workerPath + "/" + instance;
                    String instanceData = this.client.getData(instancePath);
                    if (!StringUtils.isBlank(instanceData)) {
                        JSONObject data = JSONObject.parseObject(instanceData);
                        String workerAddr = data.getString("address");
                        String workerPort = data.getString("port");
                        Integer taskNumber = data.getInteger("task.number");
                        taskNumber = null == taskNumber ? 0 : taskNumber;
                        String workerInstance = workerAddr + ":" + workerPort;
                        TaskWorker exist = this.workerRepository.exist(workerInstance, Category.Ansible.name());
                        if (null == exist) {
                            TaskWorker worker = TaskWorker.builder().instance(workerInstance).protocol("http")
                                .category(Category.Ansible).isSuper(false).build();
                            this.workerRepository.save(worker);
                            log.info("worker {} doest not exist! add it ", workerInstance);
                            log.info("数据库新增worker实例【" + workerInstance + "】");
                            exist = worker;
                        } else {
                            if (exist.getDeleted()) {
                                exist.setDeleted(false);
                                this.workerRepository.update(exist);
                                if (exist.getIsSuper()) {
                                    workerCache.addCache(CacheConstant.SUPER_WORKERS, exist.getInstance());
                                } else if (null != exist.getGroupId()) {
                                    workerCache.addCache(CacheConstant.WORKERGROUP_ + exist.getGroupId(),
                                        exist.getInstance());
                                }
                                log.info("数据库新增worker实例【" + workerInstance + "】");
                            }
                        }
                        workerStrategyCache.addCache("workmetric_" + exist.getInstance(), String.valueOf(taskNumber));
                    }
                } catch (Exception e) {
                    log.error("refresh worker exception!", e);
                }
            });
        } catch (Exception e) {
            log.error("refresh worker exception!", e);
        }
    }

    /**
     * 刷新服务实例
     *
     * @param event
     *            节点动作
     */
    private void refreshWorker(TreeCacheEvent event) {
        log.info("leader节点可监听worker节点变化，更新缓存，当前服务是否是leader：{}", service.isLeader());
        if (!service.isLeader()) {
            return;
        }
        TaskWorker worker = buildWorker(event);
        if (Objects.isNull(worker)) {
            return;
        }
        try {
            switch (event.getType()) {
                case NODE_ADDED:
                    TaskWorker exist = workerRepository.exist(worker.getInstance(), Category.Ansible.name());
                    if (null == exist) {
                        // 当前worker节点不存在，数据添加
                        worker.setCreatorId(1L);
                        this.workerRepository.save(worker);
                        log.info("worker {} doest not exist! add it ", worker.getInstance());
                        log.info("数据库新增worker实例【" + worker.getInstance() + "】");
                    } else if (exist.getDeleted()) {
                        exist.setDeleted(false);
                        this.workerRepository.update(exist);
                        if (exist.getIsSuper()) {
                            workerCache.addCache(CacheConstant.SUPER_WORKERS, exist.getInstance());
                        } else if (null != exist.getGroupId()) {
                            workerCache.addCache(CacheConstant.WORKERGROUP_ + exist.getGroupId(), exist.getInstance());
                        }
                        log.info("数据库新增worker实例【" + worker.getInstance() + "】");
                    }
                    log.info("work metric update :{} {}", worker.getInstance(), worker.getTasks());
                    workerStrategyCache.addCache("workmetric_" + worker.getInstance(),
                        String.valueOf(worker.getTasks()));
                    break;
                case NODE_REMOVED:
                    Thread.sleep(1000L);
                    String url = "http://" + worker.getInstance() + "/status";
                    RemoteService service = new RemoteService(SERVICE, new ServiceParam(), true, url, HttpMethod.GET);
                    GeneralResult<?> invoke = service.invoke();
                    if (invoke.isSuccess()) {
                        log.info("worker实例还在存活中，无需处理");
                        break;
                    }
                    TaskWorker object = workerRepository.getWorker(worker.getInstance(), Category.Ansible.name());
                    if (null != object) {
                        // 更新缓存
                        if (object.getIsSuper()) {
                            workerCache.delCache(CacheConstant.SUPER_WORKERS, object.getInstance());
                        } else if (null != object.getGroupId()) {
                            workerCache.delCache(CacheConstant.WORKERGROUP_ + object.getGroupId(),
                                    object.getInstance());
                        }
                        // 说明当前worker节点已经存在，数据删除
                        workerRepository.delete(worker.getInstance(), Category.Ansible.name());
                        log.info("数据库删除worker实例【" + worker.getInstance() + "】");
                        workerStrategyCache.delCache("workmetric_" + object.getInstance());
                        log.info("work metric remove :{}", worker.getInstance());
                    }
                    break;
                case NODE_UPDATED:
                    workerStrategyCache.addCache("workmetric_" + worker.getInstance(),
                        String.valueOf(worker.getTasks()));
                    break;
                default:
                    break;
            }
        } catch (Exception e) {
            log.error("refresh worker exception!", e);
        }
    }

    private TaskWorker buildWorker(TreeCacheEvent event) {
        if (null == event.getData() || StringUtils.isBlank(event.getData().getPath())
            || event.getData().getPath().contains("leadership")) {
            log.info("invalid path, ignore it !!");
            return null;
        }
        String instanceWorker = null;
        try {
            instanceWorker = new String(event.getData().getData(), StandardCharsets.UTF_8);
            if (!StringUtils.isBlank(instanceWorker)) {
                JSONObject data = JSONObject.parseObject(instanceWorker);
                String workerAddr = data.getString("address");
                String workerPort = data.getString("port");
                String workerInstance = workerAddr + ":" + workerPort;
                Integer taskNumber = data.getInteger("task.number");
                taskNumber = null == taskNumber ? 0 : taskNumber;
                return TaskWorker.builder().instance(workerInstance).protocol("http").category(Category.Ansible)
                    .isSuper(false).tasks(taskNumber).build();
            }
        } catch (Exception e) {
            log.error(" buildWorker error:", e);
            return null;
        }
        return null;
    }
}
