package com.bocloud.cop.booter.scheduler;

import com.bocloud.cop.entity.TaskInstanceIndex;
import com.bocloud.cop.intf.TaskInstanceIndexService;
import com.bocloud.cop.repository.TaskInstanceIndexRepository;
import com.bocloud.cop.service.utils.DateUtils;
import com.megatron.common.model.GeneralResult;
import com.megatron.framework.core.CurrentService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

/**
 * @author lyy
 * @date 2023年04月04日 10:37
 */
@Slf4j
@Component
public class TaskInstanceIndexCountScheduler {

    @Autowired
    private CurrentService service;

    @Autowired
    private TaskInstanceIndexService taskInstanceIndexService;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private TaskInstanceIndexRepository taskInstanceIndexRepository;

    /**
     * 每分钟的第30秒执行任务
     *
     * @throws Exception
     */
    @Scheduled(cron = "30 * * * * ?")
    private void countIndexMinute() throws Exception {
        if (!service.isLeader()) {
            return;
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        //统计周期(几分钟统计一次)
        long period = 1;
        LocalDateTime localDateTimeEnd = LocalDateTime.now().withSecond(0);
        LocalDateTime localDateTimeStart = localDateTimeEnd.minusMinutes(period);
        String endTime = localDateTimeEnd.format(formatter);
        String startTime = localDateTimeStart.format(formatter);
        //上次统计的结束时间
        String lastEndTime = redisTemplate.opsForValue().get("task_instance_index_time" + "_" + period + "M");
        //如果当前开始时间和上次缓存的结束时间不一样 则 缓存失效 需要查询数据库当前period最后一条数据的结束时间作为开始时间
        if (!startTime.equals(lastEndTime)) {
            //查询数据库
            TaskInstanceIndex taskInstance = taskInstanceIndexRepository.getLast(period + "M");
            //如果taskInstance为空则默认为第一次执行，如果不为空则用最后统计时间作为开始时间
            if (null != taskInstance) {
                startTime = DateUtils.dateToString(taskInstance.getCountEndTime());
            }
        } else {
            startTime = lastEndTime;
        }
        log.info("统计平台任务指标数据时间周期(1M)：开始时间-{}，结束时间-{}", startTime, endTime);
        //每一个周期记录一条记录
        while (true) {
            if(!LocalDateTime.parse(startTime,formatter).isBefore(LocalDateTime.parse(endTime,formatter))){
                break;
            }
            LocalDateTime localDateTime = LocalDateTime.parse(startTime, formatter).plusMinutes(period);
            String nextDate = localDateTime.format(formatter);
            //统计结束时间不能大于endTime
            if(LocalDateTime.parse(nextDate,formatter).isAfter(LocalDateTime.parse(endTime,formatter))){
                endTime = startTime;
                break;
            }
            GeneralResult<Void> result = taskInstanceIndexService.countIndex(startTime, nextDate, 1L, period + "M");
            if (result.isFailed()) {
                endTime = startTime;
                break;
            }
            startTime = nextDate;
        }
        redisTemplate.opsForValue().set("task_instance_index_time" + "_" + period + "M", endTime);
    }

    /**
     * 每小时的第30分钟执行任务
     *
     * @throws Exception
     */
    @Scheduled(cron = "0 30 * * * ?")
    private void countIndexHour() throws Exception {
        if (!service.isLeader()) {
            return;
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        //统计周期(几小时统计一次)
        long period = 1;
        //开始时间是当前时间往前推一个period
        LocalDateTime localDateTimeEnd = LocalDateTime.now().withMinute(0).withSecond(0);
        String endTime = localDateTimeEnd.format(formatter);
        LocalDateTime localDateTimeStart = localDateTimeEnd.minusHours(period);
        String startTime = localDateTimeStart.format(formatter);
        //查询缓存中上次统计的结束时间
        String lastEndTime = redisTemplate.opsForValue().get("task_instance_index_time" + "_" + period + "H");
        //如果当前开始时间和上次缓存的结束时间不一样 则 缓存失效 需要查询数据库当前period最后一条数据的结束时间作为开始时间
        if (!startTime.equals(lastEndTime)) {
            //查询数据库
            TaskInstanceIndex taskInstance = taskInstanceIndexRepository.getLast(period + "H");
            //如果taskInstance为空则默认为第一次执行，如果不为空则用最后统计时间作为开始时间
            if (null != taskInstance) {
                startTime = DateUtils.dateToString(taskInstance.getCountEndTime());
            }
        } else {
            startTime = lastEndTime;
        }
        log.info("统计平台任务指标数据时间周期(1H)：开始时间-{}，结束时间-{}", startTime, endTime);
        //每一个周期记录一条记录
        while (true) {
            if (!LocalDateTime.parse(startTime,formatter).isBefore(LocalDateTime.parse(endTime,formatter))) {
                break;
            }
            LocalDateTime localDateTime = LocalDateTime.parse(startTime, formatter).plusHours(period);
            String nextDate = localDateTime.format(formatter);
            //统计结束时间不能大于endTime
            if (LocalDateTime.parse(nextDate,formatter).isAfter(LocalDateTime.parse(endTime,formatter))) {
                endTime = startTime;
                break;
            }
            GeneralResult<Void> result = taskInstanceIndexService.countIndex(startTime, nextDate, 1L, period + "H");
            if (result.isFailed()) {
                endTime = startTime;
                break;
            }
            startTime = nextDate;
        }
        redisTemplate.opsForValue().set("task_instance_index_time" + "_" + period + "H", endTime);
    }

    /**
     * 每天的零点三十分执行任务
     *
     * @throws Exception
     */
    @Scheduled(cron = "0 30 0 * * ?")
    private void countIndexDay() throws Exception {
        if (!service.isLeader()) {
            return;
        }
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        //统计周期(几小时统计一次)
        long period = 1;
        //开始时间是当前时间往前推一个period
        LocalDateTime localDateTimeEnd = LocalDateTime.now().withHour(0).withMinute(0).withSecond(0);
        String endTime = localDateTimeEnd.format(formatter);
        LocalDateTime localDateTimeStart = localDateTimeEnd.minusDays(period);
        String startTime = localDateTimeStart.format(formatter);
        //查询缓存中上次统计的结束时间
        String lastEndTime = redisTemplate.opsForValue().get("task_instance_index_time" + "_" + period + "D");
        //如果当前开始时间和上次缓存的结束时间不一样 则 缓存失效 需要查询数据库当前period最后一条数据的结束时间作为开始时间
        if (!startTime.equals(lastEndTime)) {
            //查询数据库
            TaskInstanceIndex taskInstance = taskInstanceIndexRepository.getLast(period + "D");
            //如果taskInstance为空则默认为第一次执行，如果不为空则用最后统计时间作为开始时间
            if (null != taskInstance) {
                startTime = DateUtils.dateToString(taskInstance.getCountEndTime());
            }
        } else {
            startTime = lastEndTime;
        }
        log.info("统计平台任务指标数据时间周期(1D)：开始时间-{}，结束时间-{}", startTime, endTime);
        //每一个周期记录一条记录
        while (true) {
            if (!LocalDateTime.parse(startTime,formatter).isBefore(LocalDateTime.parse(endTime,formatter))) {
                break;
            }
            LocalDateTime localDateTime = LocalDateTime.parse(startTime, formatter).plusDays(period);
            String nextDate = localDateTime.format(formatter);
            //统计结束时间不能大于endTime
            if (LocalDateTime.parse(nextDate,formatter).isAfter(LocalDateTime.parse(endTime,formatter))) {
                endTime = startTime;
                break;
            }
            GeneralResult<Void> result = taskInstanceIndexService.countIndex(startTime, nextDate, 1L, period + "D");
            if (result.isFailed()) {
                endTime = startTime;
                break;
            }
            startTime = nextDate;
        }
        redisTemplate.opsForValue().set("task_instance_index_time" + "_" + period + "D", endTime);
    }
}