package com.bocloud.cop.repository;

import com.bocloud.cop.entity.Identification;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 认证DAO接口实现
 */
@Repository("identificationRepository")
public class IdentificationRepository extends BasicGenericDao<Identification, Long> {

    public IdentificationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<Identification> list(Pager pager) {
        String sql = " select a.* from identification a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, pager, "");
        return this.list(Identification.class, sql, this.getQueryBuilder().getParam(pager.getParams()));
    }

    public List<Identification> list(List<Param> params, Map<String, String> sorter) {
        String sql = " select a.id,a.username,a.account_name from identification a where a.is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "");
        return this.list(Identification.class, sql, this.getQueryBuilder().getParam(params));
    }

    public int count(List<Param> params) {
        String sql = "select count(1) from identification where is_deleted = 0";
        sql = this.getQueryBuilder().buildRaw(sql, params, null, "");
        return this.countQuery(sql, this.getQueryBuilder().getParam(params)).intValue();
    }

    public Identification query(Long id) {
        String sql = "select * from identification where id = :id";
        List<Identification> list = this.list(Identification.class, sql, MapTools.simpleMap(Common.ID, id));
        return list.isEmpty() ? null : list.get(0);
    }

    public boolean delete(Long id, Long userId) {
        String sql =
                "update identification set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", userId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 统计账号名称存在的次数
     *
     * @param accountName
     * @return
     */
    public int countAccountName(String accountName, Long id) {
        Map<String, Object> params = MapTools.simpleMap("accountName", accountName);
        String sql = " select count(1) from identification where is_deleted = 0  and account_name = :accountName ";
        if (null != id) {
            sql += " and id!=:id ";
            params.put("id", id);
        }
        return this.countQuery(sql, params).intValue();
    }

    /**
     * 根据主机资源查询关联的认证账号
     *
     * @param resourceId   资源id
     * @param resourceType 资源类型
     * @return 认证账号列表
     */
    public List<Identification> listByResource(Long resourceId, String resourceType) {
        String sql = " select * from identification i inner join  resource_identification_relation rir "
                + " on i.id = rir.identification_id  and rir.resource_type = :resourceType "
                + " where i.is_deleted  = 0 and rir.resource_id = :resourceId  ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("resourceType", resourceType);
        return this.list(Identification.class, sql, params);
    }

    /**
     * 根据ids 获取账号列表
     *
     * @param ids ids
     * @return 账号列表
     */
    public List<Identification> queryByIds(List<Long> ids) {
        String sql = "select * from identification where is_deleted = 0 and id in (:ids)";
        return this.list(Identification.class, sql, MapTools.simpleMap("ids", ids));
    }

    /**
     * 通过名称与用户名查询认证账号
     *
     * @param name     名称
     * @param userName 用户名
     * @return 认证账号信息
     */
    public Identification query(String name, String userName) {
        String sql = "select * from identification where is_deleted = 0 and account_name = :name and username = :userName";
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("userName", userName);
        List<Identification> list = this.list(Identification.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 统计名称+用户名存在的次数
     *
     * @param name     名称
     * @param userName 用户名称
     * @param id       id 排除id
     * @return 统计次数
     */
    public int count(String name, String userName, Long id) {
        Map<String, Object> params = MapTools.simpleMap("name", name);
        params.put("userName", userName);
        String sql = " select count(1) from identification where is_deleted = 0  and account_name = :name and username = :userName ";
        if (null != id) {
            sql += " and id!=:id ";
            params.put("id", id);
        }
        return this.countQuery(sql, params).intValue();
    }

    /**
     * 查询所有账号信息
     *
     * @return 所有账号信息
     */
    public List<Identification> listAll() {
        String sql = " select a.* from identification a where a.is_deleted = 0 ";
        return this.list(Identification.class, sql);
    }
}
