package com.bocloud.cop.repository;

import com.bocloud.cop.entity.ResourceIdentificationRelation;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 主机资源与认证账号的关系接口实现
 *
 * @author hongqingyu
 * @version 1.0
 * @since 2022.06.09
 */
@Repository("resourceIdentificationRelationRepository")
public class ResourceIdentificationRelationRepository extends BasicGenericDao<ResourceIdentificationRelation, Long> {

    public ResourceIdentificationRelationRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据主机资源id查询关联认证账号id列表
     * @param resourceId 主机资源id
     * @param resourceType
     * @return 认证账号id的列表
     */
    public List<Long> listIdentificationIdsByTaskHostId(Long resourceId, String resourceType) {
        String sql = "select identification_id from  resource_identification_relation "
            + " where resource_id =:resourceId and resource_type = :resourceType ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("resourceType", resourceType);
        List<ResourceIdentificationRelation> list = this.list(ResourceIdentificationRelation.class, sql, params);
        return list.stream().map(ResourceIdentificationRelation::getIdentificationId).collect(Collectors.toList());
    }

    /**
     * 根据主机资源id删除关系
     * @param resourceId 主机资源id
     * @param resourceType
     * @return 影响的关系数
     */
    public int removeByTaskHostId(Long resourceId, String resourceType) {
        String sql = "delete from resource_identification_relation "
            + " where resource_id = :resourceId and resource_type =:resourceType ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("resourceType", resourceType);
        return this.execute(sql, params);
    }

    /**
     * 根据认证账号的id删除关系
     *
     * @param identificationId
     *            认证账号id
     * @return 影响的关系数
     */
    public int removeByIdentificationId(Long identificationId) {
        String sql = "delete from resource_identification_relation where identification_id = :identificationId ";
        return this.execute(sql, MapTools.simpleMap("identificationId", identificationId));
    }

    /**
     * 根据主机资源id删除关系
     * @param resourceId 主机资源id
     * @param identificationIds 关联账号id
     * @param resourceType 资源类型
     * @return 影响的关系数
     */
    public int removeByTaskHostId(Long resourceId, List<Long> identificationIds, String resourceType) {
        String sql = "delete from resource_identification_relation where resource_id = :resourceId "
            + "  and resource_type =:resourceType  and identification_id in  (:identificationIds) ";
        Map<String, Object> params = MapTools.simpleMap("resourceId", resourceId);
        params.put("resourceType", resourceType);
        params.put("identificationIds", identificationIds);
        return this.execute(sql, params);
    }

    /**
     * 根据关联认证账号id查询主机资源id列表
     * 
     * @param resourceType
     *            资源类型
     * @param identificationId
     *            主机资源id
     * @return 认证账号id的列表
     */
    public List<Long> listTaskHostIdsByIdentificationId(Long identificationId, String resourceType) {
        String sql = "select resource_id from  resource_identification_relation "
            + " where identification_id =:identificationId and resource_type = :resourceType ";
        Map<String, Object> params = MapTools.simpleMap("identificationId", identificationId);
        params.put("resourceType", resourceType);
        List<ResourceIdentificationRelation> list = this.list(ResourceIdentificationRelation.class, sql, params);
        return list.stream().map(ResourceIdentificationRelation::getResourceId).collect(Collectors.toList());
    }

    public long count(Long identId) {
        String sql = "  select count(1) from resource_identification_relation where identification_id = :identId  ";
        return this.countQuery(sql, MapTools.simpleMap("identId", identId)).longValue();
    }

}
