package com.bocloud.cop.core;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CacheConstant;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.model.WorkerLoadInfo;
import com.bocloud.cop.model.salt.SaltCacheModel;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.util.Assert;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Slf4j
@Component
public class WorkerCache {

    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private TaskWorkerRepository taskWorkerRepository;

    private List<String> getWorkerExclude(String key) throws Exception {
        if(key.equalsIgnoreCase(CacheConstant.SUPER_WORKERS)){
            // 获取排除worker数据
            List<TaskWorker> excludeWorkers = taskWorkerRepository.listSuper(TaskWorker.Category.Ansible.name(), true);
            return excludeWorkers.stream().map(TaskWorker::getInstance).collect(Collectors.toList());
        }
        if (key.contains("workergroup_")) {
            //获取排除worker数据
            List<TaskWorker> workerExcludeList = taskWorkerRepository.listGenerals(Long.valueOf(StringUtils.substringAfterLast(key, "_")), TaskWorker.Category.Ansible.name(), true);
            return workerExcludeList.stream().map(TaskWorker::getInstance).collect(Collectors.toList());
        }
        return null;
    }

    public void addSaltCache(String key, List<TaskWorker> workers) {
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            String value = redisTemplate.opsForValue().get(key);
            SaltCacheModel info = JSONObject.parseObject(value, SaltCacheModel.class);
            log.info("新增分组【" + key + "】的worker缓存");
            if (null == info) {
                info = new SaltCacheModel();
            }
            for (TaskWorker worker : workers) {
                info.add(worker);
            }
            if (ListTool.isEmpty(info.getSaltModels())) {
                return;
            }
            redisTemplate.opsForValue().set(key, JSONObject.toJSONString(info));
        } catch (Exception e) {
            log.error("配置worker负载缓存失败:", e);
        }
    }

    public void addCache(String key, String worker) {
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            String value = redisTemplate.opsForValue().get(key);
            WorkerLoadInfo info = JSONObject.parseObject(value, WorkerLoadInfo.class);
            List<String> workerExclude = getWorkerExclude(key);
            log.info("新增分组【" + key + "】的worker缓存");
            if (null == info) {
                List<String> workers = new ArrayList<>();
                workers.add(worker);
                info = new WorkerLoadInfo(workers, 0, workerExclude);
                redisTemplate.opsForValue().set(key, JSONObject.toJSONString(info));
            } else {
                List<String> workers = info.getWorkers();
                if (!workers.contains(worker)) {
                    workers.add(worker);
                    info.setWorkers(workers);
                    info.setExcludeWorkers(workerExclude);
                    redisTemplate.opsForValue().set(key, JSONObject.toJSONString(info));
                }
            }
        } catch (Exception e) {
            log.error("配置worker负载缓存失败:", e);
        }
    }

    public void delCache(String key, String worker) {
        // 先移除掉原来分组的worker
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(60, TimeUnit.SECONDS), "请求超时");
            String value = redisTemplate.opsForValue().get(key);
            WorkerLoadInfo info = JSONObject.parseObject(value, WorkerLoadInfo.class);
            if (null == info) {
                return;
            }
            log.info("移除分组【" + key + "】的worker缓存");
            List<String> workers = info.getWorkers();
            List<String> excludeWorkers = info.getExcludeWorkers();
            if (workers.isEmpty()) {
                redisTemplate.delete(key);
                return;
            }
            if (workers.contains(worker)) {
                workers.remove(worker);
            }
            if (excludeWorkers.contains(worker)){
                excludeWorkers.remove(worker);
            }
            if (workers.isEmpty()) {
                redisTemplate.delete(key);
            } else {
                info.setWorkers(workers);
                info.setExcludeWorkers(excludeWorkers);
                redisTemplate.opsForValue().set(key, JSONObject.toJSONString(info));
            }
        } catch (Exception e) {
            log.error("配置worker负载缓存失败:", e);
        }
    }

}
