package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.module.NodeModule;
import com.bocloud.cop.entity.module.NodeModuleGroup;
import com.bocloud.cop.enums.Category;
import com.bocloud.cop.intf.NodeModuleGroupService;
import com.bocloud.cop.repository.module.NodeModuleGroupRepository;
import com.bocloud.cop.repository.module.NodeModuleRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.RequestContext.Catalog;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 编排节点类型分组抽象Service接口实现类
 */
@Slf4j
@Service("nodeModuleGroupService")
public class NodeModuleGroupServiceImpl implements NodeModuleGroupService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private NodeModuleGroupRepository nodeModuleGroupRepository;
    @Autowired
    private NodeModuleRepository nodeModuleRepository;

    @Override
    public GeneralResult<List<NodeModuleGroup>> list(Long parentId) {
        if (parentId == null) {
            // 如果不传默认为0，查根
            parentId = 0L;
        }
        try {
            List<NodeModuleGroup> groups = getGroupByPid(parentId);
            return new GeneralResult<>(true, groups, "查询模型分组成功");
        } catch (Exception e) {
            log.error("list module group fail:", e);
            return new GeneralResult<>(false, "查询模型分组失败");
        }
    }

    /**
     * 此方法递归使用,根据parentId查询子分组
     * 
     * @param parentId
     * @return
     * @throws Exception
     */
    private List<NodeModuleGroup> getGroupByPid(Long parentId) throws Exception {
        List<NodeModuleGroup> groups = nodeModuleGroupRepository.list(parentId);
        // for (NodeModuleGroup group : groups) {
        // List<NodeModuleGroup> children = getGroupByPid(group.getId());
        // if (!ListTool.isEmpty(children)) {
        // group.setChildren(children);
        // }
        // }
        return groups;
    }

    @Override
    public GeneralResult<Void> create(NodeModuleGroup group) {
        try {
            boolean flag = nodeModuleGroupRepository.exists(group.getName(), group.getParentId());
            if (flag) {
                return new GeneralResult<>(false, "此名称已存在!");
            }
            NodeModuleGroup object = nodeModuleGroupRepository.getGroup(group.getCode());
            if (null != object) {
                return new GeneralResult<>(false, "此编码已存在!");
            }
            group.setStatus("NORMAL");
            group.setInternal(false);
            nodeModuleGroupRepository.save(group);
            return new GeneralResult<>(true, "添加模型分组成功");
        } catch (Exception e) {
            log.error("Create script group fail:", e);
            return new GeneralResult<>(false, "添加模型分组失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(NodeModuleGroup group, Long userId) {
        String path = NodeModuleGroup.class.getSimpleName() + "_" + group.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NodeModuleGroup moduleGroup = nodeModuleGroupRepository.query(group.getId());
            if (null == moduleGroup) {
                log.warn("Script group data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            if (!moduleGroup.getName().equals(group.getName())) {
                boolean flag = nodeModuleGroupRepository.exists(group.getName(), group.getParentId());
                if (flag) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            if (!moduleGroup.getCode().equals(group.getCode())) {
                NodeModuleGroup object = nodeModuleGroupRepository.getGroup(group.getCode());
                if (null != object) {
                    return new GeneralResult<>(false, "此编码已存在!");
                }
            }
            moduleGroup.setName(group.getName());
            moduleGroup.setRemark(group.getRemark());
            moduleGroup.setCode(group.getCode());
            moduleGroup.setMenderId(userId);
            moduleGroup.setIcon(group.getIcon());
            nodeModuleGroupRepository.update(moduleGroup);
            return new GeneralResult<>(true, "修改模型分组成功");
        } catch (Exception e) {
            log.error("Modify script group fail:", e);
            return new GeneralResult<>(false, "修改模型分组失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = NodeModuleGroup.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NodeModuleGroup group = nodeModuleGroupRepository.query(id);
            if (null == group) {
                return new GeneralResult<>(false, "模型分组不存在");
            }
            List<NodeModuleGroup> list = nodeModuleGroupRepository.list(id);
            if (null != list && !list.isEmpty()) {
                return new GeneralResult<>(false, "模型分组存在子分组，无法删除!");
            }
            List<Param> params = Lists.newArrayList();
            if (nodeModuleRepository.count(params, id) > 0) {
                return new GeneralResult<>(false, "模型分组存在模型，无法删除!");
            }
            result = nodeModuleGroupRepository.delete(id, userId);
        } catch (Exception e) {
            log.error("Remove script group fail:", e);
            return new GeneralResult<>(false, "删除模型分组失败");
        }
        return result ? new GeneralResult<>(true, "删除模型分组成功") : new GeneralResult<>(false, "删除模型分组失败");
    }

    @Override
    public GeneralResult<NodeModuleGroup> detail(Long id) {
        try {
            NodeModuleGroup group = nodeModuleGroupRepository.query(id);
            return new GeneralResult<>(true, group, "查询模型分组详情成功");
        } catch (Exception e) {
            log.error("Query script group detail fail:", e);
            return new GeneralResult<>(false, "查询模型分组详情失败");
        }
    }

    @Override
    public GeneralResult<List<NodeModuleGroup>> listAllNodes(String code, String executeType, RequestContext context) {
        try {
            Boolean available = null;
            if (Catalog.Tenant == context.getCatalog() || Catalog.User == context.getCatalog()) {
                available = true;
            }
            List<NodeModuleGroup> groups = Lists.newArrayList();
            if (Category.Task.MULTIFUL.name().equalsIgnoreCase(code)) {
                groups.addAll(listChildren(0L, executeType, available));
            } else if (code.contains("maintain")) {
                NodeModuleGroup ctrlGroup = this.nodeModuleGroupRepository.getGroup("ctrl");
                if (null != ctrlGroup) {
                    ctrlGroup.setModules(nodeModuleRepository.list(ctrlGroup.getId(), executeType, available));
                    // ctrlGroup.setChildren(listChildren(ctrlGroup.getId(), executeType, available));
                    groups.add(ctrlGroup);
                }
                NodeModuleGroup group = this.nodeModuleGroupRepository.getGroup("maintain");
                if (null != group) {
                    List<NodeModule> modules = Lists.newArrayList();
                    if (code.contains(".")) {
                        modules.add(nodeModuleRepository.getByCode("task." + code));
                    } else {
                        modules.addAll(nodeModuleRepository.list(group.getId(), executeType, available));
                        // group.setChildren(listChildren(group.getId(), executeType, available));
                    }
                    group.setModules(modules);
                    groups.add(group);
                }
            } else if (code.contains("software")) {
                NodeModuleGroup group = this.nodeModuleGroupRepository.getGroup("software");
                if (null != group) {
                    List<NodeModule> modules = nodeModuleRepository.list(group.getId(), executeType, available);
                    // 作业编排软件组件去掉SQLServer需求，后续需要恢复去掉此行代码
                    modules.removeIf(module -> "SQLServer".equalsIgnoreCase(module.getName()));
                    group.setModules(modules);
                    // group.setChildren(listChildren(group.getId(), executeType, available));
                    groups.add(group);
                }
            } else {
                groups.addAll(getGroupByPid(0L));
            }
            return new GeneralResult<>(true, groups, "查询模型分组成功");
        } catch (Exception e) {
            log.error("list module group fail:", e);
            return new GeneralResult<>(false, "查询模型分组失败");
        }
    }

    /**
     * 此方法递归使用,根据parentId查询子分组
     * 
     * @param parentId
     * @return
     * @throws Exception
     */
    private List<NodeModuleGroup> listChildren(Long parentId, String executeType, Boolean available) throws Exception {
        List<NodeModuleGroup> list = new ArrayList<>();
        List<NodeModuleGroup> groups = nodeModuleGroupRepository.list(parentId);
        for (NodeModuleGroup group : groups) {
            // List<NodeModuleGroup> children = getGroupByPid(group.getId());
            // if (!ListTool.isEmpty(children)) {
            // group.setChildren(children);
            // }
            List<NodeModule> modules = nodeModuleRepository.list(group.getId(), executeType, available);
            if (ListTool.isEmpty(modules)) {
                continue;
            }
            // 作业编排软件组件去掉SQLServer需求，后续需要恢复去掉此行代码
            modules.removeIf(module -> "SQLServer".equalsIgnoreCase(module.getName()));
            group.setModules(modules);
            list.add(group);
        }
        return list;
    }
}
