package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.NodeTemplate;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.intf.NodeTemplateService;
import com.bocloud.cop.repository.NodeTemplateRepository;
import com.bocloud.cop.repository.TaskTemplateRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.Sign;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 作业组件模板抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2019.01.11
 */
@Slf4j
@Service("nodeTemplateService")
public class NodeTemplateServiceImpl implements NodeTemplateService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private NodeTemplateRepository nodeTemplateRepository;
    @Autowired
    private TaskTemplateRepository taskTemplateRepository;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<NodeTemplate>> list(Pager pager) {
        GridBean<NodeTemplate> gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>(4)));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = nodeTemplateRepository.count(pager.getParams());
            List<NodeTemplate> list = nodeTemplateRepository.list(pager);
            for (NodeTemplate template : list) {
                template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
        } catch (Exception e) {
            log.error("List node template fail:", e);
            return new GeneralResult<>(false, "查询组件模板失败");
        }
        return new GeneralResult<>(true, gridBean, "查询组件模板成功");
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        GridBean<SimpleBean> gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = nodeTemplateRepository.count(pager.getParams());
            List<SimpleBean> beans = nodeTemplateRepository.list(pager.getParams(), pager.getSorter());
            gridBean = new GridBean<>(1, 1, total, beans);

        } catch (Exception e) {
            log.error("List node template fail:", e);
            return new GeneralResult<>(false, "查询组件模板失败");
        }
        return new GeneralResult<>(true, gridBean, "查询组件模板成功");
    }

    @Override
    public GeneralResult<Void> create(NodeTemplate template) {
        try {
            boolean flag = nodeTemplateRepository.exists(template.getName());
            if (flag) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            nodeTemplateRepository.save(template);
            return new GeneralResult<>(true, "添加组件模板成功");
        } catch (Exception e) {
            log.error("query node template error:", e);
            return new GeneralResult<>(false, "添加组件模板失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(NodeTemplate template) {
        String path = Task.class.getSimpleName() + "_" + template.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NodeTemplate nodeTemplate = this.nodeTemplateRepository.query(template.getId());
            Assert.notNull(nodeTemplate, "数据不存在！");
            if (!nodeTemplate.getName().equals(template.getName())) {
                // 判断某种类的组件模板名称是否存在
                boolean flag = nodeTemplateRepository.exists(template.getName());
                if (flag) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
            }
            nodeTemplate.setName(template.getName());
            nodeTemplate.setRemark(template.getRemark());
            nodeTemplate.setCode(template.getCode());
            nodeTemplate.setInput(template.getInput());
            nodeTemplate.setGmtModify(new Date());
            nodeTemplateRepository.update(template);
            return new GeneralResult<>(true, "修改组件模板成功");
        } catch (Exception e) {
            log.error("Update node template fail:", e);
            return new GeneralResult<>(false, "修改组件模板失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Task.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            String value = "\"tplId\":" + id;
            List<Param> params = Lists.newArrayList();
            params.add(new Param(MapTools.simpleMap("graph", value), Sign.LK));
            int count = taskTemplateRepository.count(params);
            if (count > 0) {
                return new GeneralResult<>(false, "该组件模板已被编排模板引用无法删除");
            }
            this.nodeTemplateRepository.remove(id, userId);
            return new GeneralResult<>(true, "删除组件模板成功");
        } catch (Exception e) {
            log.error("Remove node template fail:", e);
            return new GeneralResult<>(false, "删除组件模板失败");
        }
    }

    @Override
    public GeneralResult<NodeTemplate> detail(Long id) {
        try {
            NodeTemplate template = this.nodeTemplateRepository.query(id);
            Assert.notNull(template, "数据不存在！");
            template.setCreator(redisTemplate.opsForValue().get("User_Name_" + template.getCreatorId()));
            return new GeneralResult<>(true, template, "查询组件模板详细成功");
        } catch (Exception e) {
            log.error("Query node template detail fail:", e);
            return new GeneralResult<>(false, e.getMessage());
        }
    }

}
