package com.bocloud.cop.service;

import java.util.List;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.ScriptGroup;
import com.bocloud.cop.intf.ScriptGroupService;
import com.bocloud.cop.model.ScriptGroupBean;
import com.bocloud.cop.repository.ScriptGroupRepository;
import com.bocloud.cop.repository.ScriptRepository;
import com.google.common.collect.Lists;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Param;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 脚本分组抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.16
 */
@Slf4j
@Service("scriptGroupService")
public class ScriptGroupServiceImpl implements ScriptGroupService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private ScriptGroupRepository scriptGroupRepository;
    @Autowired
    private ScriptRepository scriptRepository;

    @Override
    public GeneralResult<List<ScriptGroup>> list(Long parentId) {
        if (parentId == null) {
            // 如果不传默认为0，查根
            parentId = 0L;
        }
        try {
            List<ScriptGroup> scriptGroups = getScriptGroupByPid(parentId);
            return new GeneralResult<>(true, scriptGroups, "查询脚本分组成功");
        } catch (Exception e) {
            log.error("list scriptgroup fail:", e);
            return new GeneralResult<>(false, "查询脚本分组失败");
        }
    }

    /**
     * 此方法递归使用,根据parentId查询子分组
     * 
     * @param parentId
     * @return
     * @throws Exception
     */
    private List<ScriptGroup> getScriptGroupByPid(Long parentId) throws Exception {
        List<ScriptGroup> scriptGroups = scriptGroupRepository.list(parentId);
        for (ScriptGroup scriptGroup : scriptGroups) {
            List<ScriptGroup> children = getScriptGroupByPid(scriptGroup.getId());
            if (null != children && !children.isEmpty()) {
                scriptGroup.setChildren("[]");
                scriptGroup.setChildrenList(children);
            }
        }
        return scriptGroups;
    }

    @Override
    public GeneralResult<Void> create(ScriptGroup group) {
        try {
            boolean flag = scriptGroupRepository.exists(group.getName(), group.getParentId());
            if (flag) {
                return new GeneralResult<>(false, "此名称已存在!");
            }
            group.setStatus(BaseStatus.NORMAL.name());
            scriptGroupRepository.save(group);
            return new GeneralResult<>(true, "添加脚本分组成功");
        } catch (Exception e) {
            log.error("Create script group fail:", e);
            return new GeneralResult<>(false, "添加脚本分组失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(ScriptGroupBean bean, Long userId) {
        String path = ScriptGroup.class.getSimpleName() + "_" + bean.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ScriptGroup group = scriptGroupRepository.query(bean.getId());
            if (null == group) {
                log.warn("Script group data not exist");
                return new GeneralResult<>(false, "数据不存在");
            }
            if (!group.getName().equals(bean.getName())) {
                boolean flag = scriptGroupRepository.exists(bean.getName(), group.getParentId());
                if (flag) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            BeanUtils.copyProperties(bean, group);
            group.setMenderId(userId);
            boolean flag = scriptGroupRepository.update(group);
            return flag ? new GeneralResult<>(true, "修改脚本分组成功") : new GeneralResult<>(false, "修改脚本分组失败");
        } catch (Exception e) {
            log.error("Modify script group fail:", e);
            return new GeneralResult<>(false, "修改脚本分组失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = ScriptGroup.class.getSimpleName() + "_" + id;
        boolean result = false;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ScriptGroup group = scriptGroupRepository.query(id);
            if (null == group) {
                return new GeneralResult<>(false, "脚本分组不存在");
            }
            List<ScriptGroup> list = scriptGroupRepository.list(id);
            if (null != list && !list.isEmpty()) {
                return new GeneralResult<>(false, "脚本分组存在子分组，无法删除!");
            }
            List<Param> params = Lists.newArrayList();
            if (scriptRepository.count(params, userId, id) > 0) {
                return new GeneralResult<>(false, "脚本分组存在脚本，无法删除!");
            }
            result = scriptGroupRepository.delete(id, userId);
        } catch (Exception e) {
            log.error("Remove script group fail:", e);
            return new GeneralResult<>(false, "删除脚本分组失败");
        }
        return result ? new GeneralResult<>(true, "删除脚本分组成功") : new GeneralResult<>(false, "删除脚本分组失败");
    }

    @Override
    public GeneralResult<ScriptGroup> detail(Long id) {
        try {
            ScriptGroup group = scriptGroupRepository.query(id);
            return new GeneralResult<>(true, group, "查询脚本分组详情成功");
        } catch (Exception e) {
            log.error("Query script group detail fail:", e);
            return new GeneralResult<>(false, "查询脚本分组详情失败");
        }
    }

}
