package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.ScriptContent;
import com.bocloud.cop.entity.ScriptVersion;
import com.bocloud.cop.intf.ScriptVersionService;
import com.bocloud.cop.model.ScriptBean;
import com.bocloud.cop.repository.ScriptContentRepository;
import com.bocloud.cop.repository.ScriptRepository;
import com.bocloud.cop.repository.ScriptVersionRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 脚本版本类Service实现类
 *
 * @author wangshuai
 * @since 2011.01.11
 */
@Slf4j
@Service("scriptVersionService")
public class ScriptVersionServiceImpl implements ScriptVersionService {

    @Autowired
    private ScriptVersionRepository scriptVersionRepository;
    @Autowired
    private ScriptContentRepository scriptContentRepository;
    @Autowired
    private ScriptRepository scriptRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public GeneralResult<GridBean<ScriptVersion>> list(Pager pager) {
        List<ScriptVersion> list = null;
        int total = 0;
        GridBean<ScriptVersion> gridBean = null;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            total = scriptVersionRepository.count(pager.getParams());
            list = scriptVersionRepository.list(pager);
            for (ScriptVersion version : list) {
                version.setCreator(redisTemplate.opsForValue().get("User_Name_" + version.getCreatorId()));
                if (null != version.getMenderId()) {
                    version.setMender(redisTemplate.opsForValue().get("User_Name_" + version.getMenderId()));
                }
            }
            gridBean = new GridBean<>(pager.getPage(), pager.getRows(), total, list);
            return new GeneralResult<>(true, gridBean, "查询脚本版本成功");
        } catch (Exception e) {
            log.error("List script version fail:", e);
            return new GeneralResult<>(false, "查询脚本版本失败");
        }
    }

    @Override
    public GeneralResult<GridBean<SimpleBean>> simple(Pager pager) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = scriptVersionRepository.count(pager.getParams());
            List<SimpleBean> list = scriptVersionRepository.list(pager.getParams(), pager.getSorter());
            GridBean<SimpleBean> bean = new GridBean<>(1, 1, total, list);
            return new GeneralResult<>(true, bean, "查询脚本版本成功");
        } catch (Exception e) {
            log.error("List script version fail:", e);
            return new GeneralResult<>(false, "查询脚本版本失败");
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> create(ScriptBean bean, Long userId) {
        try {
            ScriptVersion oldVersion = scriptVersionRepository.query(bean.getVersionId());
            if (null == oldVersion) {
                log.warn("Script data not exist");
                return new GeneralResult<>(false, "脚本版本信息不存在");
            }
            Long scriptId = oldVersion.getScriptId();
            List<ScriptVersion> versions = scriptVersionRepository.listByScript(scriptId);
            for (ScriptVersion version : versions) {
                if (version.getVersion().equals(bean.getVersion())) {
                    return new GeneralResult<>(false, "该版本号已存在");
                }
            }
            // 创建新的使用版本
            ScriptVersion newVersion = new ScriptVersion();
            newVersion.setCreatorId(userId);
            newVersion.setMenderId(userId);
            newVersion.setVersion(bean.getVersion());
            newVersion.setScriptId(oldVersion.getScriptId());
            scriptVersionRepository.save(newVersion);
            // 在script_content中添加数据
            ScriptContent content = new ScriptContent();
            content.setContent(bean.getContent());
            content.setCreatorId(userId);
            content.setMenderId(userId);
            content.setVersionId(newVersion.getId());
            content.setScriptId(scriptId);
            scriptContentRepository.save(content);
            newVersion.setContentId(content.getId());
            scriptVersionRepository.update(newVersion);
            return new GeneralResult<>(true, "脚本新增版本成功");
        } catch (Exception e) {
            log.error("create script version error:", e);
            return new GeneralResult<>(false, "脚本新增版本失败");
        }
    }

    @Transactional
    @Override
    public GeneralResult<Void> modify(ScriptBean bean, Long userId) {
        String path = ScriptVersion.class.getSimpleName() + "_" + bean.getVersionId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ScriptVersion version = scriptVersionRepository.query(bean.getVersionId());
            if (null == version) {
                log.warn("Script version data not exist");
                return new GeneralResult<>(false, "脚本版本信息不存在");
            }
            Script script = scriptRepository.query(version.getScriptId());
            if (null == script) {
                log.warn("Script data not exist");
                return new GeneralResult<>(false, "脚本信息不存在");
            }
            ScriptContent oldScriptContent = scriptContentRepository.query(version.getContentId());
            if (null == oldScriptContent) {
                log.warn("Script content data not exist");
                return new GeneralResult<>(false, "脚本提交记录不存在");
            }
            String oldContent = oldScriptContent.getContent();
            if (oldContent.equals(bean.getContent())) {
                return new GeneralResult<>(false, "请修改脚本内容");
            }
            ScriptContent content = new ScriptContent();
            content.setContent(bean.getContent());
            content.setCreatorId(userId);
            content.setMenderId(userId);
            content.setVersionId(bean.getVersionId());
            content.setScriptId(version.getScriptId());
            scriptContentRepository.save(content);
            version.setContentId(content.getId());
            scriptVersionRepository.update(version);
            if (version.getVersion().equals(script.getVersion())) {
                script.setContent(bean.getContent());
                scriptRepository.update(script);
            }
            return new GeneralResult<>(true, "脚本版本修改成功");
        } catch (Exception e) {
            log.error("ScriptContent modify fail:", e);
            return new GeneralResult<>(false, "脚本版本修改失败");
        }
    }

    @Override
    public GeneralResult<Script> detail(Long id) {
        try {
            ScriptVersion version = scriptVersionRepository.query(id);
            if (null == version) {
                log.warn("Script version data not exist");
                return new GeneralResult<>(false, "查询脚本版本失败");
            }
            ScriptContent content = scriptContentRepository.query(version.getContentId());
            if (null == content) {
                log.warn("Script content data not exist");
                return new GeneralResult<>(false, "查询脚本记录失败");
            }
            Script script = scriptRepository.query(version.getScriptId());
            if (null == script) {
                log.warn("Script data not exist");
                return new GeneralResult<>(false, "查询脚本详情失败");
            }
            script.setContent(content.getContent());
            script.setVersion(version.getVersion());
            script.setVersionId(version.getId());
            return new GeneralResult<>(true, script, "查询脚本版本详情成功");
        } catch (Exception e) {
            log.error("Query script content detail fail:", e);
            return new GeneralResult<>(false, "查询脚本版本详情失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = ScriptVersion.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            ScriptVersion version = scriptVersionRepository.query(id);
            Assert.notNull(version, "数据不存在");
            if ("master".equals(version.getVersion())) {
                return new GeneralResult<>(false, "master版本不可删除");
            }
            Script script = scriptRepository.query(version.getScriptId());
            if (null == script) {
                return new GeneralResult<>(false, "脚本信息不存在");
            }
            if (id.equals(script.getVersionId())) {
                return new GeneralResult<>(false, "该版本为当前版本不可删除");
            }
            this.scriptContentRepository.removeByVersion(id, userId);
            scriptVersionRepository.delete(id, userId);
            return new GeneralResult<>(true, "脚本版本删除成功");
        } catch (Exception e) {
            log.error("Remove script fail:", e);
            return new GeneralResult<>(false, "脚本版本删除失败");
        }

    }

    @Override
    public GeneralResult<List<ScriptContent>> compare(Long id) {
        try {
            ScriptVersion noUseVersion = scriptVersionRepository.query(id);
            if (null == noUseVersion) {
                log.error("Script Version data not exist");
                return new GeneralResult<>(false, "脚本版本信息不存在");
            }
            Script script = scriptRepository.query(noUseVersion.getScriptId());
            if (null == script) {
                log.error("Script data not exist");
                return new GeneralResult<>(false, "脚本信息不存在");
            }
            ScriptVersion useVersion = scriptVersionRepository.query(script.getVersionId());
            if (null == useVersion) {
                log.error("Script Version data not exist");
                return new GeneralResult<>(false, "脚本版本信息不存在");
            }
            Long noUseId = noUseVersion.getContentId();
            Long useId = useVersion.getContentId();
            ScriptContent useContent = scriptContentRepository.query(useId);
            ScriptContent noUseContent = scriptContentRepository.query(noUseId);
            List<ScriptContent> list = new ArrayList<>();
            list.add(useContent);
            list.add(noUseContent);
            return new GeneralResult<>(true, list, "脚本对比成功");
        } catch (Exception e) {
            log.error("Version compare fail:", e);
            return new GeneralResult<>(false, "脚本对比失败");
        }
    }

}
