package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.TaskWorker;
import com.bocloud.cop.intf.ServerUserService;
import com.bocloud.cop.model.SudoArg;
import com.bocloud.cop.model.UserArg;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.WorkerLoadInfo;
import com.bocloud.cop.model.params.ServerUserCreateParams;
import com.bocloud.cop.model.params.ServerUserModifyParams;
import com.bocloud.cop.model.params.ServerUserRemoveAuthParams;
import com.bocloud.cop.model.params.ServerUserRemoveParams;
import com.bocloud.cop.model.params.ServerUserSaveAuthParams;
import com.bocloud.cop.model.result.WorkerHostResult;
import com.bocloud.cop.repository.NetworkSegmentRepository;
import com.bocloud.cop.repository.TaskWorkerRepository;
import com.bocloud.cop.service.utils.DateUtils;
import com.bocloud.cop.service.utils.OsCategoryUtils;
import com.bocloud.cop.service.utils.ParamHelper;
import com.bocloud.cop.service.utils.SshPortHelper;
import com.bocloud.cop.service.utils.worker.TaskWorkerHelper;
import com.bocloud.cop.service.utils.worker.UserTaskHelper;
import com.megatron.common.enums.OSCategory;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 主机用户管理服务实现类
 */
@Service("serverUserService")
@Slf4j
public class ServerUserServiceImpl implements ServerUserService {

    private static final String DATE_FORMAT = "yyyy/MM/dd";
    @Autowired
    private RabbitAdmin rabbitAdmin;
    @Autowired
    private AmqpTemplate amqpTemplate;
    @Autowired
    private NetworkSegmentRepository networkSegmentRepository;
    @Autowired
    TaskWorkerRepository taskWorkerRepository;
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 增加主机用户
     */
    @Override
    public GeneralResult<String> create(ServerUserCreateParams params, RequestContext context) {
        List<WorkerHost> targets = params.getTargets();
        String username = params.getUserName();
        Boolean sudo = null == params.getUserSudo() ? false : params.getUserSudo();
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        if (StringUtils.isEmpty(username)) {
            return new GeneralResult<>(false, "参数userName的值不能为空");
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            OSCategory category = OsCategoryUtils.getOSCategory(target.getCategory());
            if (category == OSCategory.AIX) {
                if (StringUtils.isEmpty(params.getCryptedType())) {
                    return new GeneralResult<>(false, "新增主机用户失败，对于AIX操作系统，必须指定密码的加密类型");
                }
            }
            target.setCategory(category.name());
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        Long expires = null;
        if (!StringUtils.isEmpty(params.getExpires())) {
            if (!DateUtils.isDate(params.getExpires(), DATE_FORMAT)) {
                return new GeneralResult<>(false, "过期时间格式错误，正确格式为：" + DATE_FORMAT);
            }
            expires = DateUtils.utcStringToDate(params.getExpires(), DATE_FORMAT).getTime() / 1000;
        }
        // worker服务需要的参数module
        // 名称
        Map<String, Object> module = new HashMap<>(16);
        module.put(Common.NAME, CommonConstant.AUTHORITY);
        // 参数
        Map<String, Object> args = new HashMap<>();
        args.put(Common.TYPE, CommonConstant.WorkerOperationType.ADD);
        UserArg user = new UserArg(username, params.getDescription(), params.getPassword(), params.getGroup(), expires,
            params.getShell(), params.getCryptedType(), false);
        if (null != params.getPasswordExpired()) {
            user.setPasswordExpired(params.getPasswordExpired());
        }
        args.put(CommonConstant.USERS, Arrays.asList(user));
        if (sudo) {
            try {
                ParamHelper.validateNotEmpty(new String[] {"hosts", "operators", "commands"},
                    new String[] {params.getHosts(), params.getOperators(), params.getCommands()});
            } catch (Exception e) {
                return new GeneralResult<>(false, e.getMessage());
            }
            SudoArg arg = new SudoArg(username, params.getHosts(), params.getCommands(), params.getOperators(),
                params.getTags(), params.getComment());
            args.put(CommonConstant.SUDO, arg);
        }
        module.put(CommonConstant.ARGS, args);
        // 选项
        Map<String, Object> options = null == params.getOptions() ? new HashMap<>() : params.getOptions();
        if (!options.isEmpty()) {
            module.put(CommonConstant.OPTIONS, options);
        }
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        String remark = "增加主机用户[" + username + "]";
        // 调用Worker服务，并将返回结果发送到站内信的消息队列
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            UserTaskHelper.invoke(instance, rabbitAdmin, amqpTemplate, paramMap, remark, context);
        } catch (Throwable t) {
            return new GeneralResult<>(false, remark + "失败，错误信息为:" + t.getMessage());
        }
        return new GeneralResult<>(true, remark + "任务提交成功");
    }

    /**
     * 修改主机用户
     */
    @Override
    public GeneralResult<String> modify(ServerUserModifyParams params, RequestContext context) {
        List<WorkerHost> targets = params.getTargets();
        String username = params.getUserName();
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        if (StringUtils.isEmpty(username)) {
            return new GeneralResult<>(false, "参数userName的值不能为空");
        }
        Long expires = null;
        if (StringUtils.isNotEmpty(params.getExpires())) {
            if (DateUtils.isDate(params.getExpires(), DATE_FORMAT)) {
                expires = DateUtils.utcStringToDate(params.getExpires(), DATE_FORMAT).getTime() / 1000;
            } else {
                return new GeneralResult<>(false, "过期时间格式错误，正确格式为：" + DATE_FORMAT);
            }
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            OSCategory category = OsCategoryUtils.getOSCategory(target.getCategory());
            if (category == OSCategory.AIX) {
                if (StringUtils.isEmpty(params.getCryptedType()) && (null != params.getPassword())) {
                    return new GeneralResult<>(false, "修改主机用户失败，对于AIX操作系统，必须指定密码的加密类型");
                }
            }
            target.setCategory(category.name());
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>(16);
        module.put(Common.NAME, CommonConstant.AUTHORITY);
        Map<String, Object> args = new HashMap<>(16);
        args.put(Common.TYPE, CommonConstant.WorkerOperationType.MODIFY);
        UserArg user = new UserArg(username, params.getDescription(), params.getPassword(), params.getGroup(), expires,
            params.getShell(), params.getCryptedType(), false);
        if (null != params.getPasswordExpired()) {
            user.setPasswordExpired(params.getPasswordExpired());
        }
        args.put(CommonConstant.USERS, Arrays.asList(user));
        module.put(CommonConstant.ARGS, args);
        Map<String, Object> options = null == params.getOptions() ? new HashMap<>() : params.getOptions();
        if (!options.isEmpty()) {
            module.put(CommonConstant.OPTIONS, options);
        }
        Map<String, Object> paramMap = new HashMap<>();
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        String remark = "修改主机用户[" + username + "]";
        // 调用Worker服务，并将返回结果发送到站内信的消息队列
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            UserTaskHelper.invoke(instance, rabbitAdmin, amqpTemplate, paramMap, remark, context);
        } catch (Throwable t) {
            return new GeneralResult<>(false, remark + "失败，错误信息为:" + t.getMessage());
        }
        return new GeneralResult<>(true, remark + "任务提交成功");
    }

    /**
     * 删除主机用户
     */
    @Override
    public GeneralResult<String> remove(ServerUserRemoveParams params, RequestContext context) {
        List<WorkerHost> targets = params.getTargets();
        String username = params.getUserName();
        Boolean force = null == params.getForce() ? false : params.getForce();
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        if (StringUtils.isEmpty(username)) {
            return new GeneralResult<>(false, "参数userName的值不能为空");
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            target.setCategory(OsCategoryUtils.getValidOSCategoryName(target.getCategory()));
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>();
        module.put(Common.NAME, CommonConstant.AUTHORITY);
        Map<String, Object> args = new HashMap<>();
        args.put(Common.TYPE, CommonConstant.WorkerOperationType.DELETE);
        UserArg user = new UserArg(username, null, null, null, 0L, null, force);
        args.put(CommonConstant.USERS, Arrays.asList(user));
        module.put(CommonConstant.ARGS, args);
        Map<String, Object> options = null == params.getOptions() ? new HashMap<>() : params.getOptions();
        if (!options.isEmpty()) {
            module.put(CommonConstant.OPTIONS, options);
        }
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        String remark = "删除主机用户[" + username + "]";
        // 调用Worker服务，并将返回结果发送到站内信的消息队列
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            UserTaskHelper.invoke(instance, rabbitAdmin, amqpTemplate, paramMap, remark, context);
        } catch (Throwable t) {
            return new GeneralResult<>(false, remark + "失败，错误信息为:" + t.getMessage());
        }
        return new GeneralResult<>(true, remark + "任务提交成功");
    }

    /**
     * 保存主机用户的sudo权限
     */
    @Override
    public GeneralResult<String> saveAuth(ServerUserSaveAuthParams params, RequestContext context) {
        List<WorkerHost> targets = params.getTargets();
        String username = params.getUserName();
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        try {
            ParamHelper.validateNotEmpty(new String[] {"userName", "hosts", "operators", "commands"},
                new String[] {username, params.getHosts(), params.getOperators(), params.getCommands()});
        } catch (Exception e) {
            return new GeneralResult<>(false, e.getMessage());
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            target.setCategory(OsCategoryUtils.getValidOSCategoryName(target.getCategory()));
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>(16);
        module.put(Common.NAME, CommonConstant.AUTHORITY);
        Map<String, Object> args = new HashMap<>(16);
        args.put(Common.TYPE, CommonConstant.WorkerOperationType.ADD);
        SudoArg sudo = new SudoArg(username, params.getHosts(), params.getCommands(), params.getOperators(),
            params.getTags(), params.getComment());
        args.put(CommonConstant.SUDO, sudo);
        module.put(CommonConstant.ARGS, args);
        // 选项
        Map<String, Object> options = null == params.getOptions() ? new HashMap<>() : params.getOptions();
        if (!options.isEmpty()) {
            module.put(CommonConstant.OPTIONS, options);
        }
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        // userName可以为用户名、也可以为用户组的名称
        String category = "主机用户[" + username + "]";
        if (username != null && username.startsWith("%")) {
            category = "用户组[" + username + "]";
        }
        String remark = "保存" + category + "的sudo权限";
        // 调用Worker服务，并将返回结果发送到站内信的消息队列
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            UserTaskHelper.invoke(instance, rabbitAdmin, amqpTemplate, paramMap, remark, context);
        } catch (Throwable t) {
            return new GeneralResult<>(false, remark + "失败，错误信息为:" + t.getMessage());
        }
        return new GeneralResult<>(true, remark + "任务提交成功");
    }

    /**
     * 删除主机用户的sudo权限
     */
    @Override
    public GeneralResult<String> removeAuth(ServerUserRemoveAuthParams params, RequestContext context) {
        List<WorkerHost> targets = params.getTargets();
        String username = params.getUserName();
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        if (StringUtils.isEmpty(username)) {
            return new GeneralResult<>(false, "参数userName的值不能为空");
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            target.setCategory(OsCategoryUtils.getValidOSCategoryName(target.getCategory()));
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>(16);
        module.put(Common.NAME, CommonConstant.AUTHORITY);
        Map<String, Object> args = new HashMap<>(16);
        args.put(Common.TYPE, CommonConstant.WorkerOperationType.DELETE);
        SudoArg sudo = new SudoArg(username, null, null, null, null, null);
        args.put(CommonConstant.SUDO, sudo);
        module.put(CommonConstant.ARGS, args);
        // 选项
        Map<String, Object> options = null == params.getOptions() ? new HashMap<>() : params.getOptions();
        if (!options.isEmpty()) {
            module.put(CommonConstant.OPTIONS, options);
        }
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        // userName可以为用户名、也可以为用户组的名称
        String category = "主机用户[" + username + "]";
        if (username != null && username.startsWith("%")) {
            category = "用户组[" + username + "]";
        }
        String remark = "删除" + category + "的sudo权限";
        // 调用Worker服务，并将返回结果发送到站内信的消息队列
        try {
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            UserTaskHelper.invoke(instance, rabbitAdmin, amqpTemplate, paramMap, remark, context);
        } catch (Throwable t) {
            return new GeneralResult<>(false, remark + "失败，错误信息为:" + t.getMessage());
        }
        return new GeneralResult<>(true, remark + "任务提交成功");
    }

    /**
     * 获取主机用户的sudo权限列表
     */
    @Override
    public GeneralResult<Map<String, String>> listAuth(WorkerHost host, String username, Boolean become,
        RequestContext context) {
        List<WorkerHost> targets = new ArrayList<>();
        targets.add(host);
        if (ListTool.isEmpty(targets)) {
            return new GeneralResult<>(false, "目标主机信息targets不能为空");
        }
        if (StringUtils.isEmpty(username)) {
            return new GeneralResult<>(false, "参数userName的值不能为空");
        }
        // worker服务需要的参数targets
        Long groupId = null;
        for (WorkerHost target : targets) {
            target.setCategory(OsCategoryUtils.getValidOSCategoryName(target.getCategory()));
            if (target.getPort() == null || target.getPort().intValue() == 0) {
                target.setPort(SshPortHelper.getDefaultPort(target.getCategory()));
            }
            // 判断主机是否存在对应的网段，分组和普通worker
            GeneralResult<Long> result = this.getGroupId(target);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            groupId = result.getData();
        }
        // worker服务需要的参数module
        Map<String, Object> module = new HashMap<>();
        module.put(Common.NAME, CommonConstant.AUTHORITY);
        Map<String, Object> args = new HashMap<>();
        args.put(Common.TYPE, CommonConstant.WorkerOperationType.VIEW);
        SudoArg sudo = new SudoArg(username, null, null, null, null, null);
        args.put(CommonConstant.SUDO, sudo);
        module.put(CommonConstant.ARGS, args);
        JSONObject options = new JSONObject();
        options.put("become", become);
        module.put(CommonConstant.OPTIONS, options);
        Map<String, Object> paramMap = new HashMap<>(16);
        paramMap.put(CommonConstant.TARGETS, targets);
        paramMap.put(CommonConstant.MODULE, module);
        try {
            // 调用同步Worker服务
            String instance = getInstance(groupId);
            if (StringUtils.isEmpty(instance)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            GeneralResult<?> result = TaskWorkerHelper.invoke(instance, paramMap, true);
            if (result.isFailed()) {
                return new GeneralResult<>(false, "查询失败，错误信息为:" + result.getMessage());
            }
            List<WorkerHostResult> results =
                JSONObject.parseArray(JSONObject.toJSONString(result.getData()), WorkerHostResult.class);
            StringBuilder builder = new StringBuilder();
            if (!ListTool.isEmpty(results)) {
                for (WorkerHostResult item : results) {
                    if (StringUtils.isNotBlank(item.getMessage())) {
                        builder.append(item.getMessage());
                    }
                }
            }
            Map<String, String> data = new HashMap<>(1);
            if (builder.length() > 0) {
                data.put(CommonConstant.CONTENT, builder.toString());
            } else {
                data.put(CommonConstant.CONTENT, "该用户或者用户组没有设置权限");
            }
            return new GeneralResult<>(true, data, "查询成功");
        } catch (Exception e) {
            return new GeneralResult<>(false, "查询失败，错误信息为:" + e.getMessage());
        }
    }

    /**
     * 判断主机是否存在对应的网段，分组和普通worker
     * 
     * @param host
     * @return
     */
    private GeneralResult<Long> getGroupId(WorkerHost host) {
        try {
            NetworkSegment segment = networkSegmentRepository.query(host.getSegmentId());
            if (null == segment) {
                return new GeneralResult<>(false, "目标机器未匹配到对应网段信息");
            }
            if (null == segment.getGroupId()) {
                return new GeneralResult<>(false, "目标机器匹配的网段未配置分组");
            }
            List<TaskWorker> workers =
                taskWorkerRepository.listGenerals(segment.getGroupId(), TaskWorker.Category.Ansible.name());
            if (ListTool.isEmpty(workers)) {
                return new GeneralResult<>(false, "网段对应分组未配置执行worker");
            }
            return new GeneralResult<>(true, segment.getGroupId(), "success");
        } catch (Exception e) {
            log.error("execute task fail:", e);
            return new GeneralResult<>(false, "网段对应分组未配置执行worker");
        }
    }

    private String getInstance(Long groupId) {
        String key = "workergroup_" + groupId;
        try (AutoCloseLock lock = lockFactory.getACLock(key)) {
            Assert.isTrue(lock.acquire(300, TimeUnit.SECONDS), "请求超时");
            String value = redisTemplate.opsForValue().get(key);
            WorkerLoadInfo loadInfo = JSONObject.parseObject(value, WorkerLoadInfo.class);
            if (null == loadInfo || ListTool.isEmpty(loadInfo.getWorkers())) {
                return null;
            }
            List<String> workers = loadInfo.getWorkers();
            int position = loadInfo.getPosition();
            if (position >= workers.size()) {
                position = 0;
            }
            String instance = workers.get(position);
            position++;
            loadInfo.setPosition(position);
            redisTemplate.opsForValue().set(key, JSONObject.toJSONString(loadInfo));
            return instance;
        } catch (Exception e) {
            log.error("配置worker负载缓存失败:", e);
            return null;
        }
    }

}
