package com.bocloud.cop.service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.NetworkSegment;
import com.bocloud.cop.entity.Script;
import com.bocloud.cop.entity.Task;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.enums.NodeCode;
import com.bocloud.cop.intf.TaskInstanceApiService;
import com.bocloud.cop.model.params.ScriptTaskParams;
import com.bocloud.cop.repository.NetworkSegmentRepository;
import com.bocloud.cop.repository.ScriptRepository;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskNodeRepository;
import com.bocloud.cop.service.utils.IpUtil;
import com.bocloud.cop.service.utils.TaskApiBuilder;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("taskInstanceApiService")
public class TaskInstanceApiServiceImpl implements TaskInstanceApiService {

    @Autowired
    private ScriptRepository scriptRepository;
    @Autowired
    private NetworkSegmentRepository networkSegmentRepository;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private TaskNodeRepository taskNodeRepository;

    @Override
    public GeneralResult<?> getResult(String instanceId) {
        try {
            TaskInstance instance = taskInstanceRepository.get(instanceId);
            if (null == instance) {
                return new GeneralResult<>(false, "任务实例不存在");
            }
            if (!"SUCCESS".equalsIgnoreCase(instance.getStatus()) && !"FAILED".equalsIgnoreCase(instance.getStatus())) {
                return new GeneralResult<>(true, "任务正在执行过程中");
            }
            List<TaskNode> nodes = taskNodeRepository.listByInstance(instance.getId());
            List<JSONObject> values = new ArrayList<>();
            for (TaskNode node : nodes) {
                if (node.getCode().equalsIgnoreCase(NodeCode.Ctrl.Start.getValue())
                    || node.getCode().equalsIgnoreCase(NodeCode.Ctrl.Stop.getValue())) {
                    continue;
                }
                // 处理输入
                JSONObject input = JSONObject.parseObject(node.getInput());
                JSONObject target = input.getJSONObject("target");
                List<JSONObject> hosts = JSONObject.parseArray(target.getString("data"), JSONObject.class);
                // 处理输出
                Map<String, JSONObject> map = new HashMap<>();
                JSONObject output = JSONObject.parseObject(node.getOutput());
                if (null != output) {
                    List<JSONObject> datas = JSONObject.parseArray(output.getString("data"), JSONObject.class);
                    if (!ListTool.isEmpty(datas)) {
                        for (JSONObject data : datas) {
                            map.put(data.getString("host"), data);
                        }
                    }
                }
                for (JSONObject host : hosts) {
                    JSONObject value = new JSONObject();
                    String key = host.getString("host");
                    if (map.containsKey(key)) {
                        JSONObject data = map.get(key);
                        value.put("host", key);
                        value.put("success", data.getBooleanValue("success"));
                        value.put("message", data.get("message"));
                    } else {
                        value.put("host", key);
                        value.put("success", null == output ? false : output.getBooleanValue("success"));
                        value.put("message", null == output ? "执行异常" : output.get("message"));
                    }
                    values.add(value);
                }
            }
            return new GeneralResult<>(true, values, "查询任务结果成功");
        } catch (Exception e) {
            log.error("query task instance result fail:", e);
            return new GeneralResult<>(false, "查询任务结果失败");
        }
    }

    @Override
    public GeneralResult<Task> task(ScriptTaskParams params) {
        try {
            // 处理没有流水号，只有目标机器的作业执行
            List<JSONObject> targets = params.getTargets();
            if (ListTool.isEmpty(targets)) {
                return new GeneralResult<>(false, "目标主机信息不能为空");
            }
            GeneralResult<Map<String, Object>> result = checkScript(params);
            if (result.isFailed()) {
                return new GeneralResult<>(false, result.getMessage());
            }
            List<NetworkSegment> segments = networkSegmentRepository.list();
            for (JSONObject target : targets) {
                for (NetworkSegment segment : segments) {
                    if (IpUtil.isInRange(target.getString("host"), segment.getName())) {
                        target.put("segmentId", segment.getId());
                        break;
                    }
                }
                if (!target.containsKey("segmentId")) {
                    return new GeneralResult<>(false, "主机未在平台上找到对应网段配置，无法进行任务操作");
                }
            }
            params.setTargets(targets);
            Task task = TaskApiBuilder.getTask(params, result.getData(), null);
            if (null == task) {
                return new GeneralResult<>(false, "作业构建失败");
            }
            return new GeneralResult<>(true, task, "作业构建成功");
        } catch (Exception e) {
            log.error("作业构建异常", e);
            return new GeneralResult<>(false, "作业构建失败");
        }
    }

    private GeneralResult<Map<String, Object>> checkScript(ScriptTaskParams params) {
        if (null == params.getScriptId() && StringUtils.isEmpty(params.getScript())) {
            return new GeneralResult<>(false, "脚本信息不能为空");
        }
        Map<String, Object> map = new HashMap<>();
        map.put("userNo", params.getUserNo());
        map.put("executeUser", params.getExecuteUser());
        try {
            String content = null;
            String category = null;
            if (null != params.getScriptId() && StringUtils.isEmpty(params.getScript())) {
                Script script = scriptRepository.get(params.getScriptId());
                if (null == script) {
                    return new GeneralResult<>(false, "平台不存在脚本内容信息");
                }
                if (!ApplyStatus.APPROVED.name().equalsIgnoreCase(script.getStatus())) {
                    return new GeneralResult<>(false, "对应脚本处在未发布状态，该脚本无法使用");
                }
                content = script.getContent();
                category = script.getCategory();
                map.put("scriptId", script.getId());
                map.put("name", script.getName());
            } else if (null == params.getScriptId() && !StringUtils.isEmpty(params.getScript())) {
                content = params.getScript();
                category = params.getType();
            } else {
                Script script = scriptRepository.get(params.getScriptId());
                if (null != script && !ApplyStatus.APPROVED.name().equalsIgnoreCase(script.getStatus())) {
                    return new GeneralResult<>(false, "对应脚本处在未发布状态，该脚本无法使用");
                }
                content = null == script ? params.getScript() : script.getContent();
                category = null == script ? params.getType() : script.getCategory();
                if (null != script) {
                    map.put("scriptId", script.getId());
                    map.put("name", script.getName());
                }
            }
            map.put("content", content);
            map.put("category", category);
            return new GeneralResult<>(true, map, "success");
        } catch (Exception e) {
            log.error("校验请求参数异常", e);
            return new GeneralResult<>(false, "校验请求参数失败");
        }
    }

    @Override
    public GeneralResult<Task> reboot(ScriptTaskParams params) {
        try {
            // 处理没有流水号，只有目标机器的作业执行
            List<JSONObject> targets = params.getTargets();
            if (ListTool.isEmpty(targets)) {
                return new GeneralResult<>(false, "目标主机信息不能为空");
            }
            List<NetworkSegment> segments = networkSegmentRepository.list();
            for (JSONObject target : targets) {
                for (NetworkSegment segment : segments) {
                    if (IpUtil.isInRange(target.getString("host"), segment.getName())) {
                        target.put("segmentId", segment.getId());
                        break;
                    }
                }
                if (!target.containsKey("segmentId")) {
                    return new GeneralResult<>(false, "主机未在平台上找到对应网段配置，无法进行任务操作");
                }
            }
            params.setTargets(targets);
            Task task = TaskApiBuilder.getRebootTask(params);
            if (null == task) {
                return new GeneralResult<>(false, "作业构建失败");
            }
            return new GeneralResult<>(true, task, "作业构建成功");
        } catch (Exception e) {
            log.error("作业构建异常", e);
            return new GeneralResult<>(false, "作业构建失败");
        }
    }

    @Override
    public GeneralResult<Task> taskTemplate(ScriptTaskParams params) {
        try {
            List<JSONObject> targets = params.getTargets();

            if (ListTool.isEmpty(targets)) {
                return new GeneralResult<>(false, "目标主机信息不能为空");
            }
            List<NetworkSegment> segments = networkSegmentRepository.list();
            for (JSONObject target : targets) {
                for (NetworkSegment segment : segments) {
                    if (IpUtil.isInRange(target.getString("host"), segment.getName())) {
                        target.put("segmentId", segment.getId());
                        break;
                    }
                }
                if (!target.containsKey("segmentId")) {
                    return new GeneralResult<>(false, "主机未在平台上找到对应网段配置，无法进行任务操作");
                }
            }

            params.setTargets(targets);
            // 依据playbook模板构建任务
            Task task = TaskApiBuilder.getTemplateTask(params);
            if (null == task) {
                return new GeneralResult<>(false, "作业构建失败");
            }
            return new GeneralResult<>(true, task, "作业构建成功");
        } catch (Exception e) {
            log.error("作业构建异常", e);
            return new GeneralResult<>(false, "作业构建失败");
        }
    }

}
