package com.bocloud.cop.service;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Base64;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.intf.TaskInstanceExportService;
import com.bocloud.cop.intf.TaskInstanceService;
import com.bocloud.cop.intf.TaskNodeService;
import com.bocloud.cop.model.result.TaskInstanceExportResult;
import com.bocloud.cop.service.utils.ExcelFactory;
import com.bocloud.cop.service.utils.TaskResulExcelHelper;
import com.bocloud.cop.service.utils.TaskResulPdfHelper;
import com.bocloud.cop.service.utils.TaskResulWordHelper;
import com.megatron.common.model.GeneralResult;

import lombok.extern.slf4j.Slf4j;

/**
 * 任务实例导出相关操作
 *
 * @author hqy
 * @version 1.0
 * @since 2022.05.13
 */
@Slf4j
@Service("taskInstanceExportService")
public class TaskInstanceExportServiceImpl implements TaskInstanceExportService {

    @Autowired
    private TaskInstanceService taskInstanceService;
    @Autowired
    private TaskNodeService taskNodeService;

    @Override
    public GeneralResult<TaskInstanceExportResult> export(Long id, String fileType) {
        // 初始化数据
        Map<String, Object> exportData = initExportData(id);
        // 构建返回
        TaskInstanceExportResult returnObj = new TaskInstanceExportResult();
        returnObj.setName((String)exportData.get("name"));
        GeneralResult<TaskInstanceExportResult> result = new GeneralResult<>();
        try {
            byte[] bytes = null;
            if ("excel".equals(fileType)) {
                bytes = exportExcel(exportData);
            } else if ("word".equals(fileType)) {
                bytes = exportWord(exportData);
            } else if ("pdf".equals(fileType)) {
                bytes = exportPdf(exportData);
            }
            returnObj.setFile(Base64.getEncoder().encodeToString(bytes));
            result.setData(returnObj);
            result.setSuccess(true);
        } catch (Exception e) {
            log.error("生成失败", e);
            result.setSuccess(false);
            result.setMessage("生成失败");
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param exportData
     *            导出数据
     * @return excel byte数组
     */
    private byte[] exportExcel(Map<String, Object> exportData) throws Exception {
        Workbook workbook = new ExcelFactory(ExcelFactory.ExcelType.EXCEL2000).newWorkbook();
        TaskResulExcelHelper exportHelper = new TaskResulExcelHelper();
        // 生成模板
        exportHelper.generateSheet(workbook, exportData);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        workbook.write(baos);
        workbook.close();
        return baos.toByteArray();
    }

    /**
     * 导出word
     *
     * @param exportData
     *            导出数据
     * @return word byte[]数组
     */
    private byte[] exportWord(Map<String, Object> exportData) throws IOException {
        TaskResulWordHelper exportHelper = new TaskResulWordHelper();
        // 生成模板
        XWPFDocument document = exportHelper.generateWord2007(exportData);
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        document.write(baos);
        document.close();
        return baos.toByteArray();
    }

    /**
     * 导出pdf
     *
     * @param exportData
     *            导出数据
     * @return pdf byte数组
     */
    private byte[] exportPdf(Map<String, Object> exportData) throws Exception {
        TaskResulPdfHelper exportHelper = new TaskResulPdfHelper();
        return exportHelper.generatePdf(exportData);
    }

    /**
     * 构建导出的数据
     *
     * @param id
     * @return
     */
    private Map<String, Object> initExportData(Long id) {
        Map<String, Object> map = new HashMap<>();
        TaskInstance taskInstance = taskInstanceService.get(id);
        List<TaskNode> nodes = taskNodeService.getNodes(id);
        map.put("name", taskInstance.getName());
        // 获取result
        Map<String, JSONArray> nodeIdToTarget = new HashMap<>();
        List<TaskNode> filteredNodes = new ArrayList<>();
        for (TaskNode node : nodes) {
            if (node.getModuleType().contains("ctrl") || "NOREADY".equals(node.getStatus())) {
                continue;
            }
            filteredNodes.add(node);
        }
        for (TaskNode node : filteredNodes) {
            String code = node.getModuleType();
            String targets = null;
            String nodeId = node.getId().toString();
            JSONObject output = JSONObject.parseObject(node.getOutput());
            if (null == output) {
                output = new JSONObject();
                output.put(CommonConstant.SUCCESS, false);
                output.put(CommonConstant.MESSAGE, "执行异常");
            }
            if (code.contains("software")
                || (code.contains("maintain") && !code.contains("method") && !code.contains(CommonConstant.DATABASE))) {
                targets = output.getString("data");
                if (!StringUtils.hasText(targets) || targets.equals("[]")) {
                    JSONObject target = new JSONObject();
                    target.put(CommonConstant.MESSAGE, output.getString(CommonConstant.MESSAGE));
                    JSONArray array = new JSONArray();
                    array.add(target);
                    targets = array.toString();
                }
            } else if (code.contains(CommonConstant.DATABASE) || code.contains("method")) {
                JSONObject target = new JSONObject();
                JSONObject input = JSONObject.parseObject(node.getInput());
                JSONObject fronts = input.getJSONObject("fronts");
                if (code.contains(CommonConstant.DATABASE)) {
                    target.put("host", fronts.get("jdbcUrl"));
                } else {
                    target.put("host", fronts.get("url"));
                }
                target.put(CommonConstant.MESSAGE, output.getString(CommonConstant.MESSAGE));
                target.put(CommonConstant.SUCCESS, output.get(CommonConstant.SUCCESS));
                JSONArray array = new JSONArray();
                array.add(target);
                targets = array.toString();
            }
            targets = Optional.ofNullable(targets).orElse("").replaceAll("<", "");
            nodeIdToTarget = initTargetMap(nodeIdToTarget, nodeId, targets);
        }
        map.put("taskInstance", taskInstance);
        map.put("taskNodes", filteredNodes);
        map.put("nodeToTarget", nodeIdToTarget);
        return map;
    }

    /**
     * 创建导出数据的node的targetMap
     *
     * @param nodeId
     * @param targets
     * @return 返回的map, key为nodeId, value为target的JsonArray
     */
    public Map<String, JSONArray> initTargetMap(Map<String, JSONArray> nodeToTarget, String nodeId, String targets) {
        JSONArray jsonArray;
        if (StringUtils.hasText(targets)) {
            jsonArray = JSONArray.parseArray(targets);
        } else {
            jsonArray = new JSONArray();
        }
        nodeToTarget.put(nodeId, jsonArray);
        return nodeToTarget;
    }
}
