package com.bocloud.cop.service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.constant.TaskStatusConstant;
import com.bocloud.cop.entity.TaskInstance;
import com.bocloud.cop.entity.TaskNode;
import com.bocloud.cop.entity.TaskNodeLog;
import com.bocloud.cop.enums.Status;
import com.bocloud.cop.intf.TaskNodeService;
import com.bocloud.cop.model.NodeControl;
import com.bocloud.cop.model.TaskGraph;
import com.bocloud.cop.model.result.HostResult;
import com.bocloud.cop.repository.NodeRelationRepository;
import com.bocloud.cop.repository.TaskInstanceRepository;
import com.bocloud.cop.repository.TaskNodeLogRepository;
import com.bocloud.cop.repository.TaskNodeRepository;
import com.bocloud.cop.service.utils.NodeHelper;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Result;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * 记录抽象Service接口实现类
 *
 * @author wei wei
 * @version 1.0
 * @since 2016.08.17
 */
@Slf4j
@Service("taskNodeService")
public class TaskNodeServiceImpl implements TaskNodeService {

    @Autowired
    private TaskNodeRepository taskNodeRepository;
    @Autowired
    private NodeRelationRepository nodeRelationRepository;
    @Autowired
    private TaskNodeLogRepository taskNodeLogRepository;
    @Autowired
    private TaskInstanceRepository taskInstanceRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<Void> modify(TaskNode node) {
        try {
            Boolean result = taskNodeRepository.update(node);
            return result ? new GeneralResult<>(true, "修改任务节点成功") : new GeneralResult<>(false, "修改任务节点失败");
        } catch (Exception e) {
            log.error("Modify job record fail:", e);
            return new GeneralResult<>(false, "修改任务节点失败");
        }
    }

    @Override
    public GeneralResult<TaskNode> detail(Long id) {
        try {
            TaskNode node = this.taskNodeRepository.query(id);
            Assert.notNull(node, "数据不存在！");
            if (null == node.getGmtFinish() || null == node.getGmtStart()) {
                node.setCost(0.00);
            } else {
                BigDecimal start = new BigDecimal(node.getGmtStart().getTime());
                BigDecimal end = new BigDecimal(node.getGmtFinish().getTime());
                node.setCost(end.subtract(start).divide(new BigDecimal(1000), 3, RoundingMode.HALF_UP).doubleValue());
            }
            List<Object> successes = Lists.newArrayList();
            List<Object> failures = Lists.newArrayList();
            if (null != node.getOutput()) {
                GeneralResult<?> result = JSONObject.parseObject(node.getOutput(), GeneralResult.class);
                if (!node.getModuleType().contains("method") && !node.getModuleType().contains("database")
                    && !node.getModuleType().contains("resource") && null != result.getData()) {
                    List<HostResult> results =
                        JSONArray.parseArray(JSONObject.toJSONString(result.getData()), HostResult.class);
                    for (HostResult hostResult : results) {
                        if (hostResult.isSuccess()) {
                            successes.add(hostResult);
                        } else {
                            failures.add(hostResult);
                        }
                    }
                }
            } else if (TaskStatusConstant.NODE_READY.equalsIgnoreCase(node.getStatus())) {
                node.setOutput(JSONObject.toJSONString(MapTools.simpleMap("message", "节点准备中···")));
            } else if (TaskStatusConstant.NODE_RUNNING.equalsIgnoreCase(node.getStatus())) {
                node.setOutput(JSONObject.toJSONString(MapTools.simpleMap("message", "节点执行中···")));
            } else {
                node.setOutput(JSONObject.toJSONString(MapTools.simpleMap("message", "没有更多了···")));
            }
            node.setSuccesses(successes);
            node.setFailures(failures);
            return new GeneralResult<>(true, node, "查询任务节点详细成功");
        } catch (Exception e) {
            log.error("Query job record detail fail:", e);
            return new GeneralResult<>(false, "查询任务节点详细失败");
        }
    }

    @Override
    public TaskNode get(Long id) {
        try {
            return taskNodeRepository.query(id);
        } catch (Exception e) {
            log.error("Get job record fail:", e);
            return null;
        }
    }

    @Override
    public List<TaskNode> getNodes(Long instanceId) {
        try {
            return this.taskNodeRepository.listByInstance(instanceId);
        } catch (Exception e) {
            log.error("Get nodes list fail:", e);
            return Lists.newArrayList();
        }
    }

    @Override
    public Result updateStatus(Long id, String status, Long userId) {
        try {
            boolean result = this.taskNodeRepository.updateStatus(id, status, userId);
            return result ? new GeneralResult<>(true, "节点状态更新成功") : new GeneralResult<>(false, "节点状态更新失败");
        } catch (Exception e) {
            log.error("update node relation status fail:", e);
            return new GeneralResult<>(false, "关系状态更新失败");
        }
    }

    @Override
    public GeneralResult<?> create(TaskNodeLog nodeLog) {
        try {
            TaskNode node = taskNodeRepository.getByUuid(nodeLog.getNodeUuid());
            if (null == node) {
                return new GeneralResult<>(false, "任务节点不存在");
            }
            nodeLog.setNodeId(node.getId());
            nodeLog.setInstanceId(node.getInstanceId());
            taskNodeLogRepository.save(nodeLog);
            return new GeneralResult<>(true, "添加任务节点日志成功");
        } catch (Exception e) {
            log.error("Create task node log fail:", e);
            return new GeneralResult<>(false, "添加任务节点日志失败");
        }
    }

    @Override
    public GeneralResult<?> getLog(Long nodeId, Integer priority, Boolean end) {
        try {
            // if ("ALL".equalsIgnoreCase(type)) {
            // List<TaskNodeLog> list = taskNodeLogRepository.list(nodeId);
            // StringBuilder builder = new StringBuilder();
            // for (TaskNodeLog log : list) {
            // builder.append(log.getContent());
            // }
            // return new GeneralResult<>(true, builder.toString(), "查询任务节点日志成功");
            // }
            if (null != end && end) {
                TaskNodeLog log = taskNodeLogRepository.get(nodeId, priority);
                if (null == log) {
                    log = new TaskNodeLog();
                    log.setLast(true);
                } else {
                    TaskNodeLog nextLog = taskNodeLogRepository.get(nodeId, priority);
                    log.setLast(null == nextLog);
                }
                if (!StringUtils.isEmpty(log.getContent())) {
                    StringBuilder builder = new StringBuilder();
                    List<String> contents = JSONObject.parseArray(log.getContent(), String.class);
                    for (String content : contents) {
                        builder.append(content + "\n");
                    }
                    log.setContent(builder.toString());
                }
                return new GeneralResult<>(true, log, "查询任务节点日志成功");
            }
            TaskNodeLog log = null;
            if (null == priority) {
                log = taskNodeLogRepository.getLast(nodeId);
                if (null == log) {
                    log = new TaskNodeLog();
                    log.setPriority(1);
                }
            } else {
                log = taskNodeLogRepository.get(nodeId, priority);
                if (null == log) {
                    log = new TaskNodeLog();
                    log.setPriority(priority);
                }
            }
            if (!StringUtils.isEmpty(log.getContent())) {
                StringBuilder builder = new StringBuilder();
                List<String> contents = JSONObject.parseArray(log.getContent(), String.class);
                for (String content : contents) {
                    builder.append(content + "\n");
                }
                log.setContent(builder.toString());
            }
            return new GeneralResult<>(true, log, "查询任务节点日志成功");
        } catch (Exception e) {
            log.error("Create job record fail:", e);
            return new GeneralResult<>(false, "查询任务节点日志失败");
        }
    }

    @Override
    public GeneralResult<TaskNode> skip(Long id, RequestContext context) {
        String path = TaskNode.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            TaskNode node = this.taskNodeRepository.query(id);
            if (null == node) {
                return new GeneralResult<>(false, "任务节点数据不存在");
            }
            TaskInstance instance = this.taskInstanceRepository.query(node.getInstanceId());
            if (null == instance) {
                return new GeneralResult<>(false, "任务实例数据不存在");
            }
            if (!Status.Inst.FAILED.name().equalsIgnoreCase(instance.getStatus())) {
                return new GeneralResult<>(false, "任务节点跳过失败：任务实例并未执行失败");
            }
            taskNodeRepository.updateFailedStatus(node.getId(), Status.Node.SKIPPED.name(), context.getTarget());
            nodeRelationRepository.updateFailedStatus(instance.getId(), node.getId(), Status.Node.SKIPPED.name());
            // 判断任务实例是否存在失败的任务节点（未配置失败跳过），如果存在判断失败节点后面是否有且只有一个判断节点
            List<TaskNode> taskNodes = taskNodeRepository.listByInstance(node.getInstanceId());
            TaskGraph graph = JSONObject.parseObject(instance.getGraph(), TaskGraph.class);
            Map<String, String> ingoreMap = NodeHelper.getIngoreNodes(graph.getNodes());
            Map<String, List<TaskNode>> nextMap = NodeHelper.getNextNodes(graph.getEdges(), ingoreMap, taskNodes);
            boolean flag = true;
            for (TaskNode taskNode : taskNodes) {
                if (Status.Node.READY.name().equalsIgnoreCase(taskNode.getStatus())) {
                    flag = false;
                    break;
                }
                if (!Status.Node.FAILED.name().equalsIgnoreCase(taskNode.getStatus())) {
                    continue;
                }
                NodeControl control = JSONObject.parseObject(taskNode.getControl(), NodeControl.class);
                if (null != control && control.isSkippable()) {
                    continue;
                }
                if (!nextMap.containsKey(taskNode.getNodeId())) {
                    flag = false;
                    break;
                }
                List<TaskNode> nextNodes = nextMap.get(taskNode.getNodeId());
                for (TaskNode nextNode : nextNodes) {
                    if (!nextNode.getModuleType().contains("ctrl.judge")) {
                        flag = false;
                        break;
                    }
                }
            }
            if (flag) {
                taskInstanceRepository.updateStatus(node.getInstanceId(), Status.Inst.SUCCESS.name(),
                    context.getTarget());
            }
            node.setStatus(Status.Node.SKIPPED.name());
            return new GeneralResult<>(true, node, "任务节点跳过成功");
        } catch (Exception e) {
            log.error("skip task node fail:", e);
            return new GeneralResult<>(false, "任务节点跳过失败");
        }
    }

}
