package com.bocloud.cop.service.compare;


import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.compare.StandardDocument;
import com.bocloud.cop.entity.compare.StandardDocumentVersion;
import com.bocloud.cop.intf.compare.StandardDocumentVersionService;
import com.bocloud.cop.repository.compare.StandardDocumentRepository;
import com.bocloud.cop.repository.compare.StandardDocumentVersionRepository;
import com.google.common.collect.Lists;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.GridHelper;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @Author liuyuanyuan
 * @Date 2022-10-292 10:55:25
 * @Description:
 */
@Slf4j
@Service("standardDocumentVersionService")
public class StandardDocumentVersionServiceImpl implements StandardDocumentVersionService {
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private StandardDocumentVersionRepository standardDocumentVersionRepository;
    @Autowired
    private StandardDocumentRepository standardDocumentRepository;
    @Value("${comparison.upload.path:/home/cmp/standard-file}")
    private String standardFilepath;

    @Override
    public GeneralResult<GridBean<StandardDocumentVersion>> list(Pager pager) {
        GridBean gridBean;
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int total = this.standardDocumentVersionRepository.count(pager.getParams());
            if (pager.getSimple()) {
                List<SimpleBean> beans = this.standardDocumentVersionRepository.list(pager.getParams(), pager.getSorter());
                List<StandardDocument> list = standardDocumentRepository.list(1, Integer.MAX_VALUE, null, null);
                Map<Long, List<StandardDocument>> documentMap = list.stream().collect(Collectors.groupingBy(StandardDocument::getVersionId));
                List<SimpleBean> resultData = beans.stream().peek(bean -> {
                    List<StandardDocument> documents = null == documentMap.get(bean.getId()) ? Lists.newArrayList() : documentMap.get(bean.getId());
                    bean.setName(bean.getName() + "(" + documents.size() + ")");
                    bean.setValue(String.valueOf(documents.size()));
                }).collect(Collectors.toList());
                gridBean = new GridBean<>(1, 1, total, resultData);
            } else {
                List<StandardDocumentVersion> list = this.standardDocumentVersionRepository.list(pager.getPage(), pager.getRows(),
                        pager.getParams(), pager.getSorter());
                gridBean = GridHelper.getBean(pager.getPage(), pager.getRows(), total, list);
            }
            return new GeneralResult<GridBean<StandardDocumentVersion>>(true, gridBean, "查询成功");
        } catch (Exception e) {
            log.error("List Query Is Fail:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }

    @Override
    public GeneralResult<Void> save(StandardDocumentVersion param, RequestContext context) {
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
            String time = dateFormat.format(new Date());
            if (null == param.getCode()) {
                param.setCode(time);
            }
            boolean resultName = standardDocumentVersionRepository.existsByName(param.getName());
            if (resultName) {
                return new GeneralResult<>(false, "此名称已存在!");
            }
            //编码会用来创建文件夹 存在对应文件
            boolean resultCode = standardDocumentVersionRepository.existsByCode(param.getCode());
            if (resultCode) {
                return new GeneralResult<>(false, "此编码已存在!");
            }
            param.setCreatorId(context.getTarget());
            param.setMenderId(context.getTarget());
            standardDocumentVersionRepository.save(param);
            return new GeneralResult<>(true, "创建成功");
        } catch (Exception e) {
            log.error("Creation failed:", e);
            return new GeneralResult<>(false, "创建失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(StandardDocumentVersion param, RequestContext context) {
        String path = StandardDocumentVersionServiceImpl.class.getSimpleName() + "_" + param.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            StandardDocumentVersion version = standardDocumentVersionRepository.query(param.getId());
            if (null == version) {
                return new GeneralResult<>(false, "数据不存在");
            }
            if(!version.getCode().equals(param.getCode())){
                return new GeneralResult<>(false, "版本编码不能修改！");
            }
            if (!version.getName().equals(param.getName())) {
                boolean result = standardDocumentVersionRepository.existsByName(param.getName());
                if (result) {
                    return new GeneralResult<>(false, "此名称已存在!");
                }
            }
            version.setName(param.getName());
            version.setRemark(param.getRemark());
            version.setGmtModify(new Date());
            version.setMenderId(context.getTarget());
            standardDocumentVersionRepository.update(version);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("Update failed:", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = StandardDocumentVersionServiceImpl.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            StandardDocumentVersion version = standardDocumentVersionRepository.query(id);
            if (null == version) {
                return new GeneralResult<>(false, "数据不存在");
            }
            List<StandardDocument> documents = standardDocumentRepository.queryByVersionId(id);
            if(!ListTool.isEmpty(documents)){
                return new GeneralResult<>(false, "先删除对应文件");
            }
            File file = new File(standardFilepath + File.separatorChar + version.getCode());
            // 判断文件是否存在
            if (!file.exists()) {
                log.error("deleteStandardFile file is not exist");
            } else {
                file.delete();
            }
            standardDocumentVersionRepository.delete(id, userId);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("Delete failed:", e);
            return new GeneralResult<>(false, "删除失败");
        }
    }

    @Override
    public GeneralResult<StandardDocumentVersion> detail(Long id) {
        try {
            StandardDocumentVersion version = standardDocumentVersionRepository.query(id);
            if (null == version) {
                return new GeneralResult<>(false, "数据不存在");
            }
            return new GeneralResult<>(true, version, "查询成功");
        } catch (Exception e) {
            log.error("Details query failed:", e);
            return new GeneralResult<>(false, "查询失败");
        }
    }
}