package com.bocloud.cop.service.inspection;

import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Base64;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.entity.Command;
import com.bocloud.cop.entity.inspection.InspectItem;
import com.bocloud.cop.entity.inspection.InspectRecord;
import com.bocloud.cop.entity.inspection.InspectRecordItem;
import com.bocloud.cop.intf.CommandCheckService;
import com.bocloud.cop.intf.inspection.ExecuteTaskService;
import com.bocloud.cop.intf.inspection.InspectRecordService;
import com.bocloud.cop.model.FastTask;
import com.bocloud.cop.model.FastTaskItemScript;
import com.bocloud.cop.model.Item;
import com.bocloud.cop.model.ScriptFront;
import com.bocloud.cop.model.Target;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.params.InspectCustomParams;
import com.bocloud.cop.model.params.ParamEntry;
import com.bocloud.cop.repository.inspection.InspectItemRepository;
import com.bocloud.cop.repository.inspection.InspectRecordItemRepository;
import com.bocloud.cop.repository.inspection.InspectRecordRepository;
import com.bocloud.cop.service.internal.CopInternalService;
import com.bocloud.cop.service.utils.GenericEntityUtil;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;

import lombok.extern.slf4j.Slf4j;

@Component
@Slf4j
public class ExecuteTaskServiceImpl implements ExecuteTaskService {

    @Autowired
    private InspectRecordService inspectRecordService;
    @Autowired
    private CopInternalService copInternalService;
    @Autowired
    private CommandCheckService commandCheckService;
    @Autowired
    private InspectItemRepository inspectItemRepository;
    @Autowired
    private InspectRecordRepository inspectRecordRepository;
    @Autowired
    private InspectRecordItemRepository inspectRecordItemRepository;

    /*
     异步执行巡检项
    */
    @Async
    public void executeItems(RunData data, Long recordId, Long userId) {
        try {
            Map<Long, Item> map = loadItemsWithDecodeContent(data.getGroups());
            String status = InspectRecord.Status.RUNNING.name();
            for (RunDataGroup group : data.getGroups()) {
                List<Item> items = new ArrayList<>();
                for (Long itemId : group.getItemIds()) {
                    Item item = map.get(itemId);
                    if (null != item) {
                        items.add(item);
                    }
                }
                if (items.size() > 0) {
                    status = executeItems(data, group, items, recordId, userId, status);
                }
            }
            log.info("try>>>" + "source::" + data.getSource() + "::recordId::" + recordId + "::Status::" + status);
            inspectRecordRepository.updateStatus(recordId, status, null, null);
        } catch (Exception e) {
            try {
                log.info("catch>>>" + "source::" + data.getSource() + "::recordId::" + recordId + "::Status::"
                    + InspectRecord.Status.ERROR.name());
                inspectRecordRepository.updateStatus(recordId, InspectRecord.Status.ERROR.name(), null, null);
            } catch (Exception ee) {
                log.error("修改记录状态失败：服务器异常：" + e);
            }
        }
        try {
            inspectRecordService.synRecord(data.getSource(), recordId);
        } catch (Exception e) {
            log.error("同步巡检记录/合规记录结果失败服务器异常：" + e);
        }
    }

    /**
     * 对group执行某个巡检项
     */
    private String executeItems(RunData data, RunDataGroup group, List<Item> items, Long recordId, Long userId,
        String status) {
        if (ListTool.isEmpty(items)) {
            return InspectRecordItem.Status.ERROR.name();
        }
        // 检查高危命令
        Map<Item, List<Command>> limitItemMap = commandCheckService.check(userId, items);
        if (MapUtils.isNotEmpty(limitItemMap)) {
            // 高危命令不能执行
            items.removeAll(limitItemMap.keySet());
            // 将错误的item日志输出
            for (Map.Entry<Item, List<Command>> entry : limitItemMap.entrySet()) {
                String contents = entry.getValue().stream().map(Command::getContent).collect(Collectors.joining(","));
                String errorMsg = "脚本中存在高危命令：[" + contents + "]";
                for (WorkerHost host : group.getTarget().getData()) {
                    saveRecordItem(group.getName(), entry.getKey(), host, recordId, null, null,
                        InspectRecordItem.Status.ERROR.name(), "", errorMsg, userId);
                }
            }
        }
        if (ListTool.isEmpty(items)) {
            return InspectRecordItem.Status.ERROR.name();
        }
        // ====构建快速任务
        List<FastTask> tasks = buildFastTask(data, group, items, userId);
        for (FastTask task : tasks) {
            try {
                // =====执行脚本
                String result = fastExecuteScript(task);
                log.info("执行脚本结果result::" + result);
                JSONObject object = JSON.parseObject(result);
                Long instanceId = object.getLong("instanceId");
                JSONObject extension = object.getJSONObject("instanceExtension");
                for (WorkerHost host : task.getTarget().getData()) {
                    for (FastTaskItemScript itemScript : task.getItemScripts()) {
                        Long nodeId = extension.getLong(itemScript.getItem().getId().toString());
                        saveRecordItem(group.getName(), itemScript.getItem(), host, recordId, instanceId, nodeId,
                            InspectRecordItem.Status.RUNNING.name(), "", "", userId);
                    }
                }
            } catch (Exception e) {
                log.error("执行job失败：服务器异常：{}", e.getMessage(), e);
                for (WorkerHost host : task.getTarget().getData()) {
                    for (FastTaskItemScript itemScript : task.getItemScripts()) {
                        saveRecordItem(group.getName(), itemScript.getItem(), host, recordId, null, null,
                            InspectRecordItem.Status.ERROR.name(), "", e.getMessage(), userId);
                    }
                }
                status = InspectRecordItem.Status.ERROR.name();
            }
        }
        return status;
    }

    private List<FastTask> buildFastTask(RunData data, RunDataGroup group, List<Item> items, Long userId) {
        List<FastTask> tasks = new ArrayList<>();
        for (WorkerHost host : group.getTarget().getData()) {
            Target target = new Target(group.getTarget().getSudo(), group.getTarget().getIdentId(),
                Collections.singletonList(host), group.getTarget().getAuthenMode(), group.getExecuteType());
            int timeout = items.stream().mapToInt(Item::getTimeout).sum();
            FastTask task = new FastTask();
            task.setCode(data.getCode());
            task.setCallback("task.job." + data.getSource().toLowerCase() + ".result");
            task.setName(data.getName() + "-" + host.getHost());
            task.setUserId(userId);
            task.setExecuteType(group.getExecuteType());
            task.setTarget(target);
            task.setTimeout(timeout);
            task.setSource(data.getSource());
            task.setItemScripts(new ArrayList<>());
            for (Item item : items) {
                // 构建ScriptFront
                ScriptFront front = new ScriptFront();
                front.setScriptId(item.getScriptId());
                front.setCategory(item.getScriptType());
                front.setContent(item.getScriptContent());
                // frontParams 为ScriptWorker中参数化过程中需要替换的字符串
                List<Map<String, String>> frontParams = new ArrayList<>();
                // 先按照巡检项目中定义的默认参数值处理 item.getScriptParam() 的值详见表t_inspect_item.script_param
                JSONArray params = JSON.parseArray(item.getScriptParam());
                if (!ListTool.isEmpty(params)) {
                    for (int i = 0; i < params.size(); i++) {
                        JSONObject param = params.getJSONObject(i);
                        Map<String, String> frontParam = new HashMap<>();
                        frontParam.put("key", param.getString("name")); // 详见ScriptWorker代码744行
                        frontParam.put("value", param.getString("value")); // 详见ScriptWorker代码744行
                        frontParams.add(frontParam);
                    }
                }
                // 再按照巡检中再次定义的参数值处理 group.getCustomParams()的取值详见t_routine_inspect_group的custom_params
                List<InspectCustomParams> customParams =
                    JSON.parseArray(group.getCustomParams(), InspectCustomParams.class);
                if (!ListTool.isEmpty(customParams)) {
                    // 过滤指定了当前巡检项目的定制化参数 或 指定了任何巡检项目的定制化参数
                    List<InspectCustomParams> filterParams = customParams.stream()
                        .filter(param -> (item.getId().equals(param.getItemId()) || 0 == param.getItemId())
                            && (host.getId().equals(param.getResourceId()) || 0 == param.getResourceId()))
                        .collect(Collectors.toList());
                    // 用定制化参数覆盖默认参数
                    for (InspectCustomParams filterParam : filterParams) {
                        if (ListTool.isEmpty(filterParam.getCustomParams())) {
                            continue;
                        }
                        for (ParamEntry entry : filterParam.getCustomParams()) {
                            frontParams.stream().filter(map -> entry.getKey().equals(map.get("key")))
                                .forEach(map -> map.put("value", entry.getValue()));
                        }
                    }
                    front.setParams(JSON.toJSONString(frontParams));
                    task.getItemScripts().add(new FastTaskItemScript(item, front));
                } else {
                    front.setParams(JSON.toJSONString(frontParams));
                    task.getItemScripts().add(new FastTaskItemScript(item, front));
                }
            }
            tasks.add(task);
        }
        return tasks;
    }

    /**
     * 执行任务
     *
     * @param task
     * @return
     * @throws RuntimeException
     */
    private String fastExecuteScript(FastTask task) {
        GeneralResult<?> result = copInternalService.patchScriptsFast(task);
        if (result.isSuccess()) {
            return result.getData().toString();
        }
        throw new RuntimeException("执行job失败:" + result.getMessage());
    }

    private Map<Long, Item> loadItemsWithDecodeContent(List<ExecuteTaskService.RunDataGroup> groups) {
        Set<Long> itemIds = new HashSet<>();
        for (ExecuteTaskService.RunDataGroup group : groups) {
            itemIds.addAll(group.getItemIds());
        }
        List<InspectItem> inspectItems = inspectItemRepository.findAllByIdIn(itemIds);
        Map<Long, Item> map = new HashMap<>();
        for (InspectItem inspectItem : inspectItems) {
            String encodeContent = inspectItem.getScriptContent();
            String decodeContent = StringUtils.isEmpty(encodeContent) ? ""
                : new String(Base64.getDecoder().decode(encodeContent.getBytes()), StandardCharsets.UTF_8);
            inspectItem.setScriptContent(decodeContent);
            Item item = new Item();
            item.setId(inspectItem.getId());
            item.setScriptContent(inspectItem.getScriptContent());
            item.setScriptParam(inspectItem.getScriptParam());
            item.setScriptType(inspectItem.getScriptType());
            item.setTimeout(inspectItem.getTimeout());
            item.setName(inspectItem.getName());
            item.setSource("inspection");
            item.setScriptId(inspectItem.getScriptId());
            map.put(inspectItem.getId(), item);
        }
        return map;
    }

    private void saveRecordItem(String groupName, Item item, WorkerHost host, Long recordId, Long jobInstanceId,
        Long jobNodeId, String status, String log, String error, Long userId) {
        InspectRecordItem recordItem = new InspectRecordItem();
        recordItem.setGroupName(groupName);
        recordItem.setItemId(item.getId());
        recordItem.setItemName(item.getName());
        recordItem.setIp(host.getHost());
        recordItem.setRecordId(recordId);
        recordItem.setJobInstanceId(jobInstanceId);
        recordItem.setJobNodeId(jobNodeId);
        recordItem.setHostId(host.getId());
        recordItem.setHostName(host.getRawName());
        recordItem.setStatus(status);
        recordItem.setLog(log);
        recordItem.setScriptErrorInfo(error);
        GenericEntityUtil.fillCreateData(recordItem, userId);
        inspectRecordItemRepository.save(recordItem);
    }
}
