package com.bocloud.cop.service.network;

import java.util.List;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.bocloud.cop.entity.NetworkGroup;
import com.bocloud.cop.intf.network.NetworkGroupService;
import com.bocloud.cop.repository.NetworkGroupRepository;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.IDFactory;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service("networkGroupService")
public class NetworkGroupServiceImpl implements NetworkGroupService {

    @Autowired
    private NetworkGroupRepository networkGroupRepository;
    @Autowired
    private LockFactory lockFactory;

    @Override
    public GeneralResult<List<NetworkGroup>> list() {
        try {
            List<NetworkGroup> groups = networkGroupRepository.list();
            return new GeneralResult<>(true, groups, "查询worker分组成功");
        } catch (Exception e) {
            log.error("list networkGroups fail:", e);
            return new GeneralResult<>(false, "查询worker分组失败");
        }
    }

    @Override
    public GeneralResult<Void> create(NetworkGroup params) {
        try {
            boolean exists = networkGroupRepository.exists(params.getName());
            if (exists) {
                return new GeneralResult<>(false, "该名称已存在，请重新输入!");
            }
            params.setProps(IDFactory.instance().uuid());
            networkGroupRepository.save(params);
            return new GeneralResult<>(true, "添加网段分组成功");
        } catch (DataBaseException e) {
            log.error("Create network group fail:", e);
            return new GeneralResult<>(false, "添加网段分组失败");
        }
    }

    @Override
    public GeneralResult<NetworkGroup> detail(Long id) {
        try {
            NetworkGroup group = this.networkGroupRepository.query(id);
            Assert.notNull(group, "网段分组数据不存在！");
            return new GeneralResult<>(true, group, "查询网段分组详细成功");
        } catch (Exception e) {
            log.error("Query network group fail:", e);
            return new GeneralResult<>(false, "查询网段分组详细失败");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = NetworkGroup.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NetworkGroup group = this.networkGroupRepository.query(id);
            Assert.notNull(group, "网段分组数据不存在！");
            boolean result = this.networkGroupRepository.delete(id, userId);
            return result ? new GeneralResult<>(true, "删除网段分组成功") : new GeneralResult<>(false, "删除网段分组失败");
        } catch (Exception e) {
            log.error("Remove network group fail:", e);
            return new GeneralResult<>(false, "删除网段分组失败");
        }
    }

    @Override
    public GeneralResult<Void> modify(NetworkGroup params) {
        String path = NetworkGroup.class.getSimpleName() + "_" + params.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            NetworkGroup bean = networkGroupRepository.query(params.getId());
            Assert.notNull(bean, "网段分组数据不存在");
            if (!bean.getName().equals(params.getName())) {
                boolean exists = networkGroupRepository.exists(params.getName());
                if (exists) {
                    return new GeneralResult<>(false, "该名称已存在，请重新输入!");
                }
                bean.setName(params.getName());
            }
            bean.setRemark(params.getRemark());
            boolean flag = networkGroupRepository.update(bean);
            return flag ? new GeneralResult<>(true, "修改网段分组成功") : new GeneralResult<>(false, "修改网段分组失败");
        } catch (Exception e) {
            log.error("Modify network group fail:", e);
            return new GeneralResult<>(false, "修改网段分组失败");
        }
    }

}
