package com.bocloud.cop.service.patch;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Optional;
import java.util.TreeSet;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import com.bocloud.cop.entity.patch.PatchDeploy;
import com.bocloud.cop.repository.patch.PatchDeployRepository;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.cop.entity.patch.Patch;
import com.bocloud.cop.entity.patch.PatchDeployHistoryDetail;
import com.bocloud.cop.enums.ApplyStatus;
import com.bocloud.cop.intf.patch.PatchDeployHistoryService;
import com.bocloud.cop.intf.patch.PatchService;
import com.bocloud.cop.model.StatResult;
import com.bocloud.cop.repository.patch.PatchRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;

import lombok.extern.slf4j.Slf4j;

/**
 * @author zoucg
 * @Company: 博云 www.bocloud.com
 * @ClassName: PatchServiceImpl
 * @Description:
 * @DATE: 2022/6/13
 */
@Slf4j
@Service("patchService")
public class PatchServiceImpl implements PatchService {

    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PatchRepository patchRepository;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private PatchDeployHistoryService patchDeployHistoryService;
    @Autowired
    private PatchDeployRepository patchDeployRepository;

    @Override
    public GeneralResult<GridBean<Patch>> list(Pager pager, Long userId) {
        try {
            pager.setParams(Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>()));
            pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(new HashMap<>()));
            pager.getSorter().put(Common.ID, Common.ONE);
            int count = patchRepository.count(pager.getParams());
            List<Patch> list = patchRepository.list(pager);
            for (Patch patch : list) {
                patch.setCreator(redisTemplate.opsForValue().get("User_Name_" + patch.getCreatorId()));
                List<PatchDeployHistoryDetail> details =
                    patchDeployHistoryService.queryByPatchIdAndStatus(patch.getId(), "success");
                List<PatchDeployHistoryDetail> detailSet = details.stream()
                    .collect(Collectors.collectingAndThen(
                        Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(t -> t.getHostId()))),
                        ArrayList::new));
                patch.setInstallCount(detailSet.size());
            }
            GridBean<Patch> bean = new GridBean<>(pager.getPage(), pager.getRows(), count, list);
            return new GeneralResult<>(true, bean, "获取数据成功！");
        } catch (Exception e) {
            log.error("List patch fail", e);
            return new GeneralResult<>(false, "获取数据失败！");
        }
    }

    @Override
    public GeneralResult<Patch> create(Patch patch, Long userId) {
        try {
            log.info("Patch create request :", patch, userId);
            verify(patch);
            build(patch, userId);
            patchRepository.save(patch);
            return new GeneralResult<>(true, "创建成功！");
        } catch (Exception e) {
            log.error("create patch error: ", e);
            return new GeneralResult<>(true, "创建失败！");
        }
    }

    @Override
    public GeneralResult<Void> remove(Long id, Long userId) {
        String path = Patch.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Patch Patch = patchRepository.get(Patch.class, id);
            if (ApplyStatus.APPROVED.name().equals(Patch.getStatus())) {
                return new GeneralResult<>(false, "选择的补丁已发布，请下架后再操作！");
            }
            // 补丁数据删除
            patchRepository.remove(Patch.class, id);
            return new GeneralResult<>(true, "删除成功");
        } catch (Exception e) {
            log.error("remove Patch fail:", e);
            return new GeneralResult<>(true, "删除失败！");
        }
    }

    @Override
    public GeneralResult<Patch> modify(Patch patch, Long userId) {
        String path = Patch.class.getSimpleName() + "_" + patch.getId();
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            Patch bean = patchRepository.get(Patch.class, patch.getId());
            if (ApplyStatus.APPROVED.name().equals(patch.getStatus())) {
                return new GeneralResult<>(false, "选择的补丁已发布，请下架后再操作！");
            }
            BeanUtils.copyProperties(patch, bean);
            build(bean, userId);
            patchRepository.update(bean);
            return new GeneralResult<>(true, "修改成功");
        } catch (Exception e) {
            log.error("modify Patch fail: ", e);
            return new GeneralResult<>(false, "修改失败");
        }
    }

    @Override
    public GeneralResult<Patch> detail(Long id) {
        try {
            Patch patch = patchRepository.get(Patch.class, id);
            return new GeneralResult<>(true, patch, "获取详情成功");
        } catch (Exception e) {
            log.error("Query Patch detail error:", e);
            return new GeneralResult<>(false, "获取详情失败," + e.getMessage());
        }
    }

    @Override
    public GeneralResult<Void> updateStatus(Long id, String status, Long userId) {
        String path = Patch.class.getSimpleName() + "_" + id;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            //下架时判断当前补丁是否被引用
            if(ApplyStatus.DEVELOPING.name().equalsIgnoreCase(status)){
                List<PatchDeploy> deploys = patchDeployRepository.listByPatchId(id);
                if(deploys.size() > 0){
                    return new GeneralResult<>(false, "状态更新失败:补丁已被补丁部署【" + deploys.get(0).getName() + "】引用无法下架" );
                }
            }
            boolean result = patchRepository.updateStatus(id, status, userId);
            return result ? new GeneralResult<>(true, "操作成功") : new GeneralResult<>(false, "操作失败");
        } catch (Exception e) {
            log.error("updateStatus Patch fail:", e);
            return new GeneralResult<>(false, "操作失败");
        }
    }

    @Override
    public GeneralResult<List<StatResult>> statsStatus(RequestContext context) {
        Pager pager = new Pager();
        pager.setRows(Integer.MAX_VALUE);
        List<Patch> patchList = patchRepository.list(pager);
        int developing = 0;
        int approved = 0;
        for (Patch patch : patchList) {
            if (ApplyStatus.DEVELOPING.name().equalsIgnoreCase(patch.getStatus())) {
                developing++;
                continue;
            } else if (ApplyStatus.APPROVED.name().equalsIgnoreCase(patch.getStatus())) {
                approved++;
                continue;
            }
        }
        List<StatResult> results = new ArrayList<>();
        results.add(new StatResult("未发布", developing));
        results.add(new StatResult("已发布", approved));
        return new GeneralResult<>(true, results, "获取执行状态统计成功");
    }

    private void verify(Patch patch) {
        log.info("verify tpatch :", patch);
        if (StringUtils.isEmpty(patch.getName())) {
            throw new IllegalArgumentException("补丁名称不能为空");
        }
        if (StringUtils.isEmpty(patch.getPatchNo())) {
            throw new IllegalArgumentException("补丁编号不能为空");
        }
        if (StringUtils.isEmpty(patch.getScriptContent())) {
            throw new IllegalArgumentException("脚本不能为空");
        }
    }

    private void build(Patch patch, Long userId) {
        log.info(" Patch build :", patch, userId);
        patch.setDeleted(false);
        if (null == patch.getId()) {
            patch.setGmtCreate(new Date());
            patch.setCreatorId(userId);
        }
        patch.setGmtModify(new Date());
        patch.setMenderId(userId);
        patch.setStatus(ApplyStatus.DEVELOPING.name());
        patch.setFiles(JSONArray.toJSONString(patch.getFiles()));
    }

}
