package com.bocloud.cop.service.utils;

import static java.util.concurrent.TimeUnit.DAYS;
import static java.util.concurrent.TimeUnit.HOURS;
import static java.util.concurrent.TimeUnit.MILLISECONDS;
import static java.util.concurrent.TimeUnit.MINUTES;
import static java.util.concurrent.TimeUnit.SECONDS;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.util.Date;
import java.util.TimeZone;
import java.util.concurrent.TimeUnit;

/**
 * 日期工具类
 */
public final class DateUtils {

    /**
     * 获取两个时间之间的相差的时间
     * <p>
     * 选择合适的时间单位，舍入计算后所得数值的所有位数为4位，例如：1.000 ns , 1.235 ms
     */
    public static String getElapsedTimeWith4Digits(Date begin, Date end) {
        long millis = end.getTime() - begin.getTime();
        TimeUnit unit = chooseUnit(millis);
        double value = (double)millis / MILLISECONDS.convert(1, unit);
        return String.format("%.4g", value) + chineseUinit(unit);
    }

    private static TimeUnit chooseUnit(long millis) {
        if (DAYS.convert(millis, MILLISECONDS) > 0) {
            return DAYS;
        }
        if (HOURS.convert(millis, MILLISECONDS) > 0) {
            return HOURS;
        }
        if (MINUTES.convert(millis, MILLISECONDS) > 0) {
            return MINUTES;
        }
        if (SECONDS.convert(millis, MILLISECONDS) > 0) {
            return SECONDS;
        }
        return MILLISECONDS;
    }

    private static String chineseUinit(TimeUnit unit) {
        switch (unit) {
            case NANOSECONDS:
                return "纳秒";
            case MICROSECONDS:
                return "微秒";
            case MILLISECONDS:
                return "毫秒";
            case SECONDS:
                return "秒";
            case MINUTES:
                return "分钟";
            case HOURS:
                return "小时";
            case DAYS:
                return "天";
            default:
                return "未知的时间单位";
        }
    }

    /**
     * 将字符串转成时间
     */
    public static Date stringToDate(String dateString, String format) {
        if (dateString == null || "".equals(dateString)) {
            throw new RuntimeException("传入参数中的[时间串]不能为空！");
        }
        if (format == null || "".equals(format)) {
            throw new RuntimeException("传入参数中的[时间格式]不能为空！");
        }
        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串
        Date date;
        try {
            date = df.parse(dateString);
        } catch (ParseException e) {
            throw new RuntimeException(e.getMessage());
        }
        return date;
    }

    /**
     * 将字符串转成时间
     */
    public static Date stringToDate(String dateString) {
        if (dateString == null || "".equals(dateString)) {
            throw new RuntimeException("传入参数中的[时间串]不能为空！");
        }
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串
        Date date;
        try {
            date = df.parse(dateString);
        } catch (ParseException e) {
            throw new RuntimeException(e.getMessage());
        }
        return date;
    }

    /**
     * 将时间转成字符串
     */
    public static String dateToString(Date date, String format) {
        if (date == null) {
            return null;
        }
        if (format == null || "".equals(format)) {
            throw new RuntimeException("传入参数中的[时间格式]不能为空！");
        }
        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串
        return df.format(date);
    }

    /**
     * 将时间转成字符串
     */
    public static String dateToString(Date date) {
        if (date == null) {
            return null;
        }
        SimpleDateFormat df = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        df.setLenient(false);// 将2017-13-01看做是一个非法的日期字符串
        return df.format(date);
    }

    /**
     * 判断某个字符串是否为有效的日期
     */
    public static boolean isDate(String dateString, String format) {
        SimpleDateFormat df = new SimpleDateFormat(format);
        // 将2017-13-01看做是一个非法的日期字符串
        df.setLenient(false);
        try {
            df.parse(dateString);
            return true;
        } catch (ParseException e) {
            return false;
        }
    }

    /**
     * 将UTC时间字符串转成时间
     */
    public static Date utcStringToDate(String dateString, String format) {
        if (dateString == null || "".equals(dateString)) {
            throw new RuntimeException("传入参数中的[时间串]不能为空！");
        }
        if (format == null || "".equals(format)) {
            throw new RuntimeException("传入参数中的[时间格式]不能为空！");
        }
        SimpleDateFormat df = new SimpleDateFormat(format);
        df.setLenient(false);
        df.setTimeZone(TimeZone.getTimeZone("UTC"));
        Date date;
        try {
            date = df.parse(dateString);
        } catch (ParseException e) {
            throw new RuntimeException(e.getMessage());
        }
        return date;
    }

    /**
     * LocalDateTime 转 Date
     * @param localDateTime
     * @return
     */
    public static Date localDateTimeToDate(LocalDateTime localDateTime){
        if(null == localDateTime){
            return null;
        }
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }

    /**
     * Date 转 LocalDateTime
     * @param date
     * @return
     */
    public static LocalDateTime dateToLocalDateTime(Date date){
        if(null == date){
            return null;
        }
        Instant instant = date.toInstant();
        return instant.atZone(ZoneId.systemDefault()).toLocalDateTime();
    }

    /**
     * LocalDate 转 Date
     * @param localDate
     * @return
     */
    public static Date localDateToDate(LocalDate localDate){
        if(null == localDate){
            return null;
        }
        LocalDateTime localDateTime = LocalDateTime.of(localDate, LocalTime.now());
        return Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
    }



}
