package com.bocloud.cop.service.utils;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cop.service.internal.SmsInternalService;
import com.google.common.collect.Lists;
import com.megatron.common.encrypt.DESEncryptor;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.JSONTools;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.registry.RegistryClient;
import jakarta.mail.MessagingException;
import jakarta.mail.internet.MimeMessage;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.exception.ExceptionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.MailException;
import org.springframework.mail.javamail.JavaMailSenderImpl;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * 邮件发送器
 *
 * @author dmw
 */
@Slf4j
@Component("emailSender")
public class EmailSender {

    private final RegistryClient client;
    private final SmsInternalService smsInternalService;
    private String emailServer = "";
    private String userName = "";
    private String password = "";
    private JavaMailSenderImpl sender;

    @Autowired
    public EmailSender(RegistryClient client, SmsInternalService smsInternalService) {
        this.client = client;
        this.smsInternalService = smsInternalService;
    }

    private void send(String subject, String[] receiver, String mailContent, List<File> files) {
        sender = new JavaMailSenderImpl();
        try {
            // 获取zk上最新的邮箱配置信息
            String data = this.client.getSystem();
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                emailServer = object.getString("EmailServer");
                userName = object.getString("EmailUserName");
                password = new DESEncryptor().decrypt(object.getString("EmailPassword"), "BocloudCMP");
            }
        } catch (Exception exception) {
            log.error("GET ZOOKEEPER DATE ERR:{}", ExceptionUtils.getStackTrace(exception));
        }
        if (StringUtils.isEmpty(emailServer) || StringUtils.isEmpty(userName) || null == password) {
            log.warn("email config error,unable to send mail");
            return;
        }
        sender.setHost(emailServer);
        sender.setUsername(userName);
        if (!StringUtils.isEmpty(password)) {
            sender.setPassword(password);
        }
        // ssl端口号
        int port = 465;
        // 当发件人服务器为腾讯企业邮箱时需开启认证
        if (emailServer.contains("qq")) {
            sender.setPort(port);
            Properties properties = new Properties();
            // 开启认证
            properties.setProperty("mail.smtp.auth", "true");
            // 启用调试
            properties.setProperty("mail.debug", "false");
            // 设置链接超时
            properties.setProperty("mail.smtp.timeout", "25000");
            // 设置端口
            properties.setProperty("mail.smtp.port", Integer.toString(port));
            // 设置ssl端口
            properties.setProperty("mail.smtp.socketFactory.port", Integer.toString(port));
            properties.setProperty("mail.smtp.socketFactory.fallback", "false");
            properties.setProperty("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
            properties.setProperty("mail.smtp.ssl.enable", "true");
            sender.setJavaMailProperties(properties);
        } else {
            port = 25;
            sender.setPort(port);
        }
        log.info("sender server:{}", emailServer);
        log.info("sender port:{}", port);
        log.info("send email:{}", userName);
        List<String> receivers = Lists.newArrayList();
        Collections.addAll(receivers, receiver);
        String sendHostName = StringUtils.substringBefore(StringUtils.substringAfter(userName, "@"), ".");
        Iterator<String> iterator = receivers.iterator();
        while (iterator.hasNext()) {
            String email = iterator.next();
            String receiverHostName = StringUtils.substringBefore(StringUtils.substringAfter(email, "@"), ".");
            if (sendHostName.equals(receiverHostName)) {
                try {
                    checkEmailAndSend(subject, email, files, null, mailContent);
                } catch (MessagingException e) {
                    log.error("Email send exception: {}", ExceptionUtils.getStackTrace(e));
                } finally {
                    iterator.remove();
                }
            }
        }
        try {
            if (receivers.size() > 0) {
                checkEmailAndSend(subject, null, files, receivers.toArray(new String[0]), mailContent);
            }
        } catch (MailException e) {
            log.error("Email send exception: {}", ExceptionUtils.getStackTrace(e));
        } catch (MessagingException e) {
            log.error("Email sender MessagingException: {}", ExceptionUtils.getStackTrace(e));
        }
    }

    private void checkEmailAndSend(String subject, String receiver, List<File> files, String[] receivers,
        String mailContent) throws MessagingException {
        // 建立HTML邮件消息
        MimeMessage mailMessage = sender.createMimeMessage();
        // true表示开始附件模式
        MimeMessageHelper messageHelper = new MimeMessageHelper(mailMessage, true, "utf-8");
        // 设置收件人，寄件人
        if (null != receiver) {
            messageHelper.setTo(receiver);
        } else {
            messageHelper.setTo(receivers);
        }
        messageHelper.setFrom(userName);
        messageHelper.setSubject(subject);
        if (!ListTool.isEmpty(files)) {
            for (File file : files) {
                messageHelper.addAttachment(file.getName(), file);
            }
        }
        // true 表示启动HTML格式的邮件
        String text = "<html><head></head><body>" + mailContent + "</h1></body></html>";
        log.info("邮件内容:{}", text);
        messageHelper.setText("<html><head></head><body>" + mailContent + "</h1></body></html>", true);
        Date beginDate = new Date(System.currentTimeMillis());
        // 发送邮件
        try {
            sender.send(mailMessage);
        } catch (MailException e) {
            log.error("Email sender MessagingException: {}", ExceptionUtils.getStackTrace(e));
            log.error("Error Email: {}", "<html><head></head><body>" + mailContent + "</h1></body></html>");
            log.error(
                "========Email send begin========" + new SimpleDateFormat("yyyy年MM月dd日_HH:mm:ss").format(beginDate));
        }
        Date endDate = new Date(System.currentTimeMillis());
        log.info("========Email send end========" + new SimpleDateFormat("yyyy年MM月dd日_HH:mm:ss").format(endDate));
    }

    public boolean testConnected() {
        try {
            sender = new JavaMailSenderImpl();
            // 获取zk上最新的邮箱配置信息
            String data = this.client.getSystem();
            JSONObject object = JSONTools.isJSONObj(data);
            if (null != object) {
                emailServer = object.getString("EmailServer");
                userName = object.getString("EmailUserName");
                password = new DESEncryptor().decrypt(object.getString("EmailPassword"), "BocloudCMP");
            }
            sender.setHost(emailServer);
            sender.setUsername(userName);
            if (!StringUtils.isEmpty(password)) {
                sender.setPassword(password);
            }
            // ssl端口号
            int port = 465;
            // 当发件人服务器为腾讯企业邮箱时需开启认证
            if (emailServer.contains("qq")) {
                sender.setPort(port);
                Properties properties = new Properties();
                // 开启认证
                properties.setProperty("mail.smtp.auth", "true");
                // 启用调试
                properties.setProperty("mail.debug", "false");
                // 设置链接超时
                properties.setProperty("mail.smtp.timeout", "25000");
                // 设置端口
                properties.setProperty("mail.smtp.port", Integer.toString(port));
                // 设置ssl端口
                properties.setProperty("mail.smtp.socketFactory.port", Integer.toString(port));
                properties.setProperty("mail.smtp.socketFactory.fallback", "false");
                properties.setProperty("mail.smtp.socketFactory.class", "javax.net.ssl.SSLSocketFactory");
                properties.setProperty("mail.smtp.ssl.enable", "true");
                sender.setJavaMailProperties(properties);
            } else {
                port = 25;
                sender.setPort(port);
            }
            this.sender.testConnection();
        } catch (Exception e) {
            log.error("get mail connection error:", e);
            return false;
        }
        return true;
    }

    public void send(JSONObject message, List<File> files, List<RequestContext> contexts) {
        try {
            // 收件邮箱
            for (RequestContext context : contexts) {
                GeneralResult<JSONObject> result = smsInternalService.getUser(context.getTarget());
                if (result.isFailed()) {
                    continue;
                }
                JSONObject object = result.getData();
                if (object != null && StringUtils.isNotEmpty(object.getString("email"))) {
                    send(message.getString("subject"), new String[] {object.getString("email")},
                        message.getString("content"), files);
                }
            }
        } catch (Exception e) {
            log.error("Send message or email error:", e);
        }
    }
}