package com.bocloud.cop.service.utils;

import com.bocloud.cop.entity.Identification;
import com.bocloud.cop.repository.IdentificationRepository;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.*;

import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * 认证账号Excel导入工具
 *
 * @author hqy
 * @date 2023/02/23 11:04
 */
@Slf4j
public class IdentificationExcelHelper {
    private IdentificationRepository identificationRepository;

    public IdentificationExcelHelper() {
        super();
    }

    public IdentificationExcelHelper(IdentificationRepository identificationRepository) {
        super();
        this.identificationRepository = identificationRepository;
    }

    /**
     * 生成模板
     *
     * @param workbook
     *            表格信息
     */
    public void generateSheet(Workbook workbook) {
        Sheet sheet = workbook.createSheet("认证账号");
        Row row0 = sheet.createRow(0);
        row0.setHeight((short)400);
        Row row1 = sheet.createRow(1);
        row1.setHeight((short)400);
        // 定义颜色
        CellStyle cellStyle = workbook.createCellStyle();
        // 居中
        cellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        cellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        cellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        cellStyle.setBorderRight(BorderStyle.THIN);
        Font font = workbook.createFont();
        font.setFontName("仿宋_GB2312");
        // 粗体显示
        font.setBold(true);
        font.setFontHeightInPoints((short)12);
        cellStyle.setFont(font);

        CellStyle bodyCellStyle = workbook.createCellStyle();
        // 居中
        bodyCellStyle.setAlignment(HorizontalAlignment.CENTER);
        // 下边框
        bodyCellStyle.setBorderBottom(BorderStyle.THIN);
        // 左边框
        bodyCellStyle.setBorderLeft(BorderStyle.THIN);
        // 右边框
        bodyCellStyle.setBorderRight(BorderStyle.THIN);
        Font bodyFont = workbook.createFont();
        bodyFont.setFontName("仿宋_GB2312");
        bodyFont.setFontHeightInPoints((short)11);
        bodyCellStyle.setFont(bodyFont);
        // 生成表头
        generateHeader(sheet, cellStyle, row0);
        // 设置文本格式以及调整列宽,使之能完整显示字段名称
        ExcelHelper.setTextformatAndColumnwidth(workbook);
    }

    /**
     * 生成表头
     *
     * @param sheet
     *            表单信息
     * @param cellStyle
     *            单元格样式
     * @param row0
     *            行号
     */
    private void generateHeader(Sheet sheet, CellStyle cellStyle, Row row0) {
        Cell cell0;
        // 设置主机excel表头属性信息
        List<String> properties = fileHeaderProperties();
        int index = 0;
        for (String property : properties) {
            cell0 = row0.createCell(index);
            cell0.setCellValue(new ExcelFactory(row0).newRichTextString(property));
            cell0.setCellStyle(cellStyle);
            sheet.setColumnWidth(index, 60 * 256);
            index++;
        }
    }

    public static List<String> fileHeaderProperties() {
        List<String> properties = new ArrayList<>();
        properties.add("名称*");
        properties.add("用户名*");
        properties.add("密码*");
        return properties;
    }

    private void validateHeader(Map<String, Integer> header, List<String> properties) throws Exception {
        //判断标题字段的未知是否相同
        for (String name : header.keySet()) {
            if(properties.contains(name) && !header.get(name).equals(properties.indexOf(name))){
                throw new Exception("表头中字段位置不匹配:"+name);
            }
        }
        Set<String> propSetImport = new HashSet<>(header.keySet());
        Set<String> propSetNew = new HashSet<>(properties);
        Set<String> propSetNewTemp = new HashSet<>(propSetNew);
        propSetNewTemp.removeAll(propSetImport);
        if (!propSetNewTemp.isEmpty()) {
            throw new Exception("表头中缺少以下属性：" + propSetNewTemp.toString());
        }
    }

    private void validataWb(Workbook wb) throws Exception {
        List<String> properties = fileHeaderProperties();
        Sheet sheet = wb.getSheetAt(0);
        Map<String, Integer> header = readHeader4NameKey(sheet.getRow(0));
        validateHeader(header, properties);
    }

    public Map<String, Integer> readHeader4NameKey(Row row) {
        if (row == null) {
            throw new RuntimeException("1、请检查是否是最新版本;\n" + "2、不要更改模板格式;\n" + "3、请重新下载模板;");
        }
        Map<String, Integer> header = new HashMap<>(30);
        try {
            for (int i = 0; i < getLastCellNum(row); i++) {
                Cell cell = row.getCell(i);
                // 有错误信息栏目
                if (cell != null && !"".equals(ExcelHelper.getCellValue(cell))) {
                    String value = ExcelHelper.getCellValue(cell);
                    header.put(value, i);
                }
            }
        } catch (Exception e) {
            log.error("检查表头报错", e);
            throw new RuntimeException("1、请检查是否是最新版本;\n" + "2、不要更改模板格式;\n" + "3、请重新下载模板;\n");
        }

        return header;
    }

    private int getLastCellNum(Row row) {
        // 最后一个有效单元格的位置
        short lastNum = 0;
        if (row.getLastCellNum() < BoStringUtils.parseInt(ExcelHelper.EnumHelper.ExcelColumnMax.getValue())) {
            return row.getLastCellNum();
        }
        for (int i = 0; i < BoStringUtils.parseInt(ExcelHelper.EnumHelper.ExcelColumnMax.getValue()); i++) {
            // 如果当前单元格不是null 并且有值
            if (!"".equals(ExcelHelper.getCellValue(row.getCell(i)))) {
                lastNum = (short)i;
            }
        }
        return lastNum;
    }

    /**
     * 校验数据
     *
     * @param wb
     *            导入表格
     * @return 校验结果
     */
    private GeneralResult<Void> checkData(Workbook wb) {
        log.info("开始读取认证账号Excel数据");
        try {
            boolean checkResult = true;
            String message = "";
            Sheet sheet = wb.getSheetAt(0);
            if (null == sheet) {
                log.error("导入错误:EXCEL缺少SHEET");
                return new GeneralResult<>(false, "导入错误:EXCEL缺少SHEET");
            }
            List<String> keys = new ArrayList<>();
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                Row row = sheet.getRow(i);
                if (row == null) {
                    continue;
                }
                Cell nameCell = row.getCell(0);
                String name = null;
                if (null != nameCell) {
                    nameCell.setCellType(CellType.STRING);
                    name = String.valueOf(nameCell);
                }
                Cell userNameCell = row.getCell(1);
                String userName = null;
                if (null != userNameCell) {
                    userNameCell.setCellType(CellType.STRING);
                    userName = String.valueOf(userNameCell);
                }
                Cell pwdCell = row.getCell(2);
                String pwd = null;
                if (null != pwdCell) {
                    pwdCell.setCellType(CellType.STRING);
                    pwd = String.valueOf(pwdCell);
                }
                if (StringUtils.isBlank(name) || StringUtils.isBlank(userName) || StringUtils.isBlank(pwd)) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行数据不全";
                    log.warn(message);
                    break;
                }
                if (keys.contains(name + "_" + userName)) {
                    checkResult = false;
                    message = "第" + (i + 1) + "行名称用户名已经存在";
                    log.warn(message);
                    break;
                }
                keys.add(name + "_" + userName);
            }
            return new GeneralResult<>(checkResult, message);
        } catch (Exception e) {
            log.error("认证账号数据检查失败:", e);
            return new GeneralResult<>(false, "认证账号数据检查失败:" + e.getMessage());
        }

    }

    /**
     * 导入数据存在
     *
     * @param wb
     *            表格
     * @param context
     *            操作人信息
     */
    private GeneralResult<Void> saveData(Workbook wb, RequestContext context) {
        log.info("开始保存认证账号");
        try {
            Sheet sheet = wb.getSheetAt(0);
            Map<String, Identification> identMap = identificationRepository.listAll().stream().collect(Collectors.toMap(
                ident -> ident.getAccountName() + "_" + ident.getUsername(), Function.identity(), (t1, t2) -> t2));
            List<Identification> addList = new ArrayList<>();
            List<Identification> updateList = new ArrayList<>();
            for (int i = 1; i <= sheet.getLastRowNum(); i++) {
                Row row = sheet.getRow(i);
                if(null == row){
                    continue;
                }
                Cell nameCell = row.getCell(0);
                nameCell.setCellType(CellType.STRING);
                String name = String.valueOf(nameCell);
                Cell userNameCell = row.getCell(1);
                userNameCell.setCellType(CellType.STRING);
                String userName = String.valueOf(userNameCell);
                Cell pwdCell = row.getCell(2);
                pwdCell.setCellType(CellType.STRING);
                String pwd = String.valueOf(pwdCell);
                Identification ident = identMap.get(name + "_" + userName);
                // 存在的更新，不存在的新增
                if (null == ident) {
                    ident = new Identification();
                    ident.setAccountName(name);
                    ident.setPassword(TaskHostDataUtil.aesEncryptPwd(pwd));
                    ident.setUsername(userName);
                    ident.setDeleted(false);
                    ident.setCreatorId(context.getTarget());
                    addList.add(ident);
                } else {
                    ident.setPassword(TaskHostDataUtil.aesEncryptPwd(pwd));
                    ident.setMenderId(context.getTarget());
                    updateList.add(ident);
                }
            }
            if (addList.size() > 0) {
                identificationRepository.batchSave(addList, 1000);
            }
            if (updateList.size() > 0) {
                identificationRepository.batchUpdate(updateList, 1000);
            }
            log.info("认证账号批量导入结束");
            return new GeneralResult<>(true, "认证账号批量导入成功");
        } catch (Exception e) {
            log.error("认证账号导入失败:", e);
            return new GeneralResult<>(false, "认证账号导入失败:" + e.getMessage());
        }
    }

    /**
     * excel数据处理
     *
     * @param wb
     *            导入表格
     * @param context
     *            操作人信息
     * @return 操作结果
     */
    public GeneralResult<Void> processExcel(Workbook wb, RequestContext context) {
        try {
            validataWb(wb);
            GeneralResult<Void> result = checkData(wb);
            if (result.isFailed()) {
                return result;
            }
            return saveData(wb, context);
        } catch (Exception e) {
            log.error("导入错误:EXCEL校验失败");
            return new GeneralResult<>(false, "导入错误:EXCEL校验失败");
        }
    }

}
