package com.bocloud.cop.service.utils.worker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.Callable;

import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.model.WorkerHost;
import com.bocloud.cop.model.result.WorkerHostResult;
import com.megatron.framework.http.model.RemoteService;

/**
 * 调用异步Worker的实现类
 */
@Slf4j
public final class AsyncWorkerCaller implements Callable<GeneralResult<List<WorkerHostResult>>> {

    private RemoteService remoteService;
    private Map<String, Object> params;
    private RabbitAdmin rabbitAdmin;
    private Long timeout;

    public AsyncWorkerCaller(RemoteService remoteService, Map<String, Object> params, RabbitAdmin rabbitAdmin,
        Long timeout) {
        this.remoteService = remoteService;
        this.params = params;
        this.rabbitAdmin = rabbitAdmin;
        this.timeout = null == timeout ? 60 * 60 * 1000L : timeout * 60 * 1000L;
    }

    @Override
    @SuppressWarnings("unchecked")
    public GeneralResult<List<WorkerHostResult>> call() {
        List<WorkerHost> targets = (List<WorkerHost>)params.get(CommonConstant.TARGETS);
        if (!remoteService.isExist()) {
            List<WorkerHostResult> taskResults = new ArrayList<>(targets.size());
            for (WorkerHost target : targets) {
                taskResults.add(
                    new WorkerHostResult(target.getId(), false, target.getHost(), target.getUser(), "远程Worker服务不存在"));
            }
            return new GeneralResult<>(false, taskResults, "远程Worker服务不存在");
        }

        // 创建一个接收消息的临时队列
        String queueId = UUID.randomUUID().toString();
        rabbitAdmin.declareQueue(new Queue(queueId, false, false, true));
        rabbitAdmin.declareBinding(new Binding(queueId, Binding.DestinationType.QUEUE,
            BoCloudExchange.Default.DIRECT.getExchange(), queueId, null));
        // 指定参数response的值为"sync"，Worker服务就会直接返回操作结果
        Map<String, Object> params = (Map<String, Object>)remoteService.getParam().getParams();
        if (null == params) {
            params = new HashMap<String, Object>();
        }
        params.put(CommonConstant.QUEUE, queueId);
        params.put(CommonConstant.RESPONSE, CommonConstant.WorkerResponseType.ASYNC);
        remoteService.getParam().setParams(params);
        remoteService.setDataFormat(PostDataFormat.RAW);
        Map<String, Object> module = (Map<String, Object>)params.get(CommonConstant.MODULE);
        log.info("start to async call remote worker to execute [{}] task", module.get(Common.NAME));
        GeneralResult<?> result = remoteService.invoke();
        if (!result.isSuccess()) {
            log.error("invoke remote worker [{}] failed for : {}", remoteService.getUrl(), result.getMessage());
            return buildErrorResult(result.getMessage(), targets, result.getMessage());
        }
        try {
            Message message = rabbitAdmin.getRabbitTemplate().receive(queueId, timeout);
            if (message == null) {
                log.error("获取任务执行结果消息超时！");
                return buildErrorResult("获取任务结果消息超时！", targets, "获取任务结果消息超时！");
            }
            String content = new String(message.getBody(), "UTF-8");
            log.info("get result [{}] from remote worker", content);
            JSONObject object = JSONObject.parseObject(content);
            if (!object.getBooleanValue("success")) {
                return buildErrorResult(object.getString("data"), targets, "执行任务失败");
            }
            List<WorkerHostResult> results = JSONArray.parseArray(object.getString("data"), WorkerHostResult.class);
            if (ListTool.isEmpty(results)) {
                results = new ArrayList<>();
                for (WorkerHost target : targets) {
                    results.add(new WorkerHostResult(target.getId(), object.getBooleanValue("success"),
                        target.getHost(), target.getUser(), object.getString("message")));
                }
            }
            return new GeneralResult<>(object.getBooleanValue("success"), results, object.getString("message"));
        } catch (Exception e) {
            log.error("等待Worker【{}】返回任务结果消息异常:", remoteService.getUrl(), e);
            return buildErrorResult(e.getMessage(), targets, e.getMessage());
        } finally {
            rabbitAdmin.deleteQueue(queueId);
        }
    }

    private static GeneralResult<List<WorkerHostResult>> buildErrorResult(String content, List<WorkerHost> targets,
        String message) {
        GeneralResult<List<WorkerHostResult>> jobResult = new GeneralResult<>(false, message);
        List<WorkerHostResult> taskResults = new ArrayList<>();
        JSONArray results = null;
        try {
            results = JSONArray.parseArray(content);
        } catch (Exception e) {
            log.error("将调用Worker服务返回的字符串转化为json对象出错。(返回的字符串为 " + content + ")");
        }
        if (!ListTool.isEmpty(results)) {
            for (Object result : results) {
                JSONObject item = (JSONObject)result;
                taskResults.add(new WorkerHostResult(item.getLong("id"), item.getBoolean("success"),
                    item.getString("host"), item.getString("user"), item.getString("message")));
            }
        } else {
            for (WorkerHost target : targets) {
                taskResults
                    .add(new WorkerHostResult(target.getId(), false, target.getHost(), target.getUser(), message));
            }
        }
        jobResult.setData(taskResults);
        return jobResult;
    }

}
