package com.bocloud.cop.service.utils.worker;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorCompletionService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.model.NotifyMessage;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.ListTool;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import com.megatron.framework.http.model.ServiceParam;

import lombok.extern.slf4j.Slf4j;

/**
 * 调用Worker服务工具类
 */
@Slf4j
public class UserTaskHelper extends WorkerHelper {

    protected static final ExecutorService HANDLER_JOBRESULT_EXECUTOR =
        Executors.newFixedThreadPool(Runtime.getRuntime().availableProcessors());

    private static final Service SERVICE = Service.create(BoCloudService.GAIA.getName());

    /**
     * 调用Worker服务，并将返回结果发送到站内信的消息队列
     */
    public static void invoke(ServiceFactory serviceFactory, RabbitAdmin rabbitAdmin, AmqpTemplate amqpTemplate,
        Map<String, Object> paramMap, String remark, RequestContext context) {
        ExecutorService threadQueue = Executors.newSingleThreadExecutor();
        ExecutorCompletionService<GeneralResult<?>> completionService = new ExecutorCompletionService<>(threadQueue);
        RemoteService remoteService =
            serviceFactory.build(SERVICE, JOB_SUBMIT_URL, HttpMethod.OPERATE, HEADERS, paramMap);
        completionService.submit(new WorkerCaller(remoteService, rabbitAdmin, amqpTemplate, paramMap));
        handlerJobResult(completionService, remark, context, amqpTemplate);
    }

    /**
     * 调用Worker服务，并将返回结果发送到站内信的消息队列
     */
    public static void invoke(String instance, RabbitAdmin rabbitAdmin, AmqpTemplate amqpTemplate,
        Map<String, Object> params, String remark, RequestContext context) {
        ExecutorService threadQueue = Executors.newSingleThreadExecutor();
        ExecutorCompletionService<GeneralResult<?>> completionService = new ExecutorCompletionService<>(threadQueue);
        String url = "http://" + instance + JOB_SUBMIT_URL;
        RemoteService service =
            new RemoteService(SERVICE, new ServiceParam(params, HEADERS), true, url, HttpMethod.POST);
        service.setDataFormat(PostDataFormat.RAW);
        completionService.submit(new WorkerCaller(service, rabbitAdmin, amqpTemplate, params));
        handlerJobResult(completionService, remark, context, amqpTemplate);
    }

    /**
     * 将返回结果发送到站内信的消息队列
     */
    private static void handlerJobResult(ExecutorCompletionService<GeneralResult<?>> completionService,
        String taskDescription, RequestContext context, AmqpTemplate amqpTemplate) {
        HANDLER_JOBRESULT_EXECUTOR.execute(() -> {
            try {
                GeneralResult<?> result = completionService.take().get();
                handleFunc(result, taskDescription, context, amqpTemplate);
            } catch (Throwable t) {
                log.error("worker任务执行异常", t);
                // 发送站内信
                String subject = taskDescription + "执行结果";
                String msg = "worker任务执行异常" + t.getMessage();
                NotifyMessage message = new NotifyMessage(context, subject, msg, true, false);
                amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(),
                    CommonConstant.MESSAGE_QUEUE_ROUTINGKEY, message);
            }
        });
    }

    @SuppressWarnings("unchecked")
    private static void handleFunc(GeneralResult<?> result, String taskDescription, RequestContext context,
        AmqpTemplate amqpTemplate) {
        List<TaskTargetResult> results = (List<TaskTargetResult>)result.getData();
        // 忽略无返回结果的记录
        if (ListTool.isEmpty(results)) {
            if (result.isSuccess()) {
                taskDescription = taskDescription + "执行成功";
            } else {
                taskDescription = taskDescription + "执行失败";
            }
            // 发送站内信
            String subject = taskDescription + "执行结果";
            NotifyMessage message = new NotifyMessage(context, subject, taskDescription, true, false);
            amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), CommonConstant.MESSAGE_QUEUE_ROUTINGKEY,
                message);
            return;
        }
        StringBuilder sb = new StringBuilder();
        int size = results.size();
        int index = 0;
        for (TaskTargetResult tResult : results) {
            if (StringUtils.isEmpty(tResult.getHost())) {
                continue;
            }
            sb.append("主机[" + tResult.getHost() + "]_用户[" + tResult.getUser() + "]执行");
            if (tResult.getSuccess()) {
                sb.append("成功");
            } else {
                sb.append("失败，原因：");
                sb.append(tResult.getErrorMsg() == null ? "" : tResult.getErrorMsg());
            }
            if (index++ < size - 1) {
                sb.append("；");
            }
        }
        // 发送站内信
        String subject = taskDescription + "执行结果";
        NotifyMessage message = new NotifyMessage(context, subject, sb.toString(), true, false);
        amqpTemplate.convertAndSend(BoCloudExchange.Default.DIRECT.getExchange(), CommonConstant.MESSAGE_QUEUE_ROUTINGKEY,
            message);
    }

}
