package com.bocloud.cop.service.utils.worker;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.Callable;

import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitAdmin;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.megatron.common.enums.PostDataFormat;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.utils.ListTool;
import com.bocloud.cop.constant.CommonConstant;
import com.bocloud.cop.model.WorkerHost;
import com.megatron.framework.http.model.RemoteService;

/**
 * 调用Worker的实现类
 */
@Slf4j
public final class WorkerCaller implements Callable<GeneralResult<?>> {
    /**
     * 接收rabbitmq消息的超时时间
     */
    public static final long TIMEOUT_MILLIS = 1000L * 60 * 30;

    private RemoteService remoteService;
    private RabbitAdmin rabbitAdmin;
    private AmqpTemplate amqpTemplate;
    private Map<String, Object> paramMap;
    private long timeoutMillis = TIMEOUT_MILLIS;

    public WorkerCaller(RemoteService remoteService, RabbitAdmin rabbitAdmin, AmqpTemplate amqpTemplate,
        Map<String, Object> paramMap) {
        this.remoteService = remoteService;
        this.rabbitAdmin = rabbitAdmin;
        this.amqpTemplate = amqpTemplate;
        this.paramMap = paramMap;

        if (paramMap.containsKey(CommonConstant.TIMEOUT)) {
            // 单位：秒
            long timeout = (Long)paramMap.get(CommonConstant.TIMEOUT);
            if (timeout > 0) {
                setTimeoutMillis(1000L * timeout);
            }
        }
    }

    public void setTimeoutMillis(long timeoutMillis) {
        this.timeoutMillis = timeoutMillis;
    }

    @Override
    public GeneralResult<?> call() throws Exception {
        if (!remoteService.isExist()) {
            List<WorkerHost> failedTargets =
                JSONObject.parseArray(JSONObject.toJSONString(paramMap.get(CommonConstant.TARGETS)), WorkerHost.class);
            List<TaskTargetResult> taskResults = new ArrayList<>(failedTargets.size());
            for (WorkerHost target : failedTargets) {
                taskResults.add(new TaskTargetResult(false, target.getHost(), target.getUser(), null, "远程Worker服务不存在"));
            }
            return new GeneralResult<>(false, taskResults, "远程Worker服务不存在");
        }

        // 创建一个接收消息的临时队列
        String queueId = UUID.randomUUID().toString();
        rabbitAdmin.declareQueue(new Queue(queueId, false, false, true));
        rabbitAdmin.declareBinding(new Binding(queueId, Binding.DestinationType.QUEUE,
            BoCloudExchange.Default.DIRECT.getExchange(), queueId, null));

        // 指定队列名称
        Map<String, Object> params = (Map<String, Object>)remoteService.getParam().getParams();
        if (null == params) {
            params = new HashMap<String, Object>();
        }
        params.put(CommonConstant.QUEUE, queueId);
        remoteService.getParam().setParams(params);
        remoteService.setDataFormat(PostDataFormat.RAW);

        GeneralResult<?> invoke = remoteService.invoke();
        if (!invoke.isSuccess()) {
            return buildErrorResult(invoke.getMessage(), remoteService, invoke.getMessage());
        }
        GeneralResult<?> result;
        try {
            Message message = amqpTemplate.receive(queueId, timeoutMillis);
            if (message == null) {
                log.error("获取任务执行结果消息超时！");
                return buildErrorResult("获取任务结果消息超时！", remoteService, "获取任务结果消息超时！");
            }
            String content = new String(message.getBody());
            JSONObject jsonObject = JSONObject.parseObject(content);
            Boolean isSuccess = jsonObject.getBoolean("success");

            if (!isSuccess) {
                return buildErrorResult(jsonObject.getString("data"), remoteService, "执行任务失败");
            }
            List<TaskTargetResult> taskResults =
                JSONArray.parseArray(jsonObject.getString("results"), TaskTargetResult.class);
            result = new GeneralResult<>(jsonObject.getBooleanValue("success"), taskResults,
                jsonObject.getString("message"));

        } catch (Exception e) {
            result = buildErrorResult(e.getMessage(), remoteService, e.getMessage());
        } finally {
            rabbitAdmin.deleteQueue(queueId);
        }
        return result;
    }

    /**
     * 组装错误信息
     */
    private static GeneralResult<?> buildErrorResult(String value, RemoteService remoteService, String message) {
        GeneralResult<List<TaskTargetResult>> result = new GeneralResult<>(false, message);
        List<TaskTargetResult> results = new ArrayList<>();
        JSONArray array = null;
        try {
            array = JSONArray.parseArray(value);
        } catch (Exception e) {
            log.error("将调用Worker服务返回的字符串转化为json对象出错。(返回的字符串为 " + value + ")");
        }
        if (array != null && !ListTool.isEmpty(array)) {
            for (Object object : array) {
                JSONObject item = (JSONObject)object;
                results.add(new TaskTargetResult(item.getBoolean("success"), item.getString("host"),
                    item.getString("user"), null, item.getString("message")));
            }
        } else {
            Map<String, Object> params = (Map<String, Object>)remoteService.getParam().getParams();
            if (null == params) {
                params = new HashMap<String, Object>();
            }
            List<WorkerHost> targets =
                JSONObject.parseArray(JSONObject.toJSONString(params.get(CommonConstant.TARGETS)), WorkerHost.class);
            for (WorkerHost target : targets) {
                results.add(new TaskTargetResult(false, target.getHost(), target.getUser(), null, value));
            }
        }
        result.setData(results);
        return result;
    }
}
